# Copyright(c) 1998, Space Science and Engineering Center, UW-Madison
# Refer to "McIDAS Software Acquisition and Distribution Policies"
# in the file  mcidas/data/license.txt

# $Id: ImageSelectlite.tcl,v 1.55 2004/06/18 20:50:13 tsmith Exp $

#-------------------------------------------------------------------------------#
# proc CreateDataset {directory_list area_file_list stand_modis_file_list 
#		      file_name directory} 
#	- calls DSSERVE to create the necessary datasets using
#	  the directory supplied for the file mask.  It then 
#	  calls ImageSelect supplying it with the appropriate
#	  dataset name and image position.
#-------------------------------------------------------------------------------#
proc CreateDataset {directory_list area_file_list stand_modis_file_list \
		    file_name directory} \
{
   # Get index of file in list box
   set num_directories [llength $directory_list]

   # Set the local dataset name
   set file_index [lsearch $area_file_list $file_name]
   if {$file_index >= 0} \
   {
      	set dataset "MCLITE1"
	incr file_index
   } \
   else \
   {
	set file_index [lsearch $stand_modis_file_list $file_name]

	# If file is not in AREA format then check standard MODIS file list
   	if {$file_index >= 0} \
   	{
	   set dataset "MCLITE2"
	   incr file_index
	} \
	else \
	{
	   tk_messageBox \
		-message "You must make a selection first." \
		-icon error \
		-title "Error"
	   return
	}
   }

   # Create datasets (MCLITE1 is for all AREA format files, MCLITE2 
   #                  is for standard MODIS files)
   mcidas exec "DSSERVE ADD LOCAL/MCLITE1 AREA DEV=NNN DIRF='$directory/*' \
		\"AREA files for McIDAS-Lite"
   mcidas exec "DSSERVE ADD LOCAL/MCLITE2 MODS DEV=NNN \
		DIRF='$directory/MOD02*' TYPE=IMAGE \
		\"Standard MODIS files for McIDAS-Lite"

   # Kill ImageSelection window if it already exists
     if { [winfo exists .tlImageSelect] } {destroy .tlImageSelect}

   # Send the dataset and position to ImageSelect
     ImageSelect $file_name LOCAL $dataset $file_index
}



#-----------------------------------------------------------------------#
# proc ImageSelect { file_name group dataset position } 
#		-- Upper level code to organize the IMGDISP command
#-----------------------------------------------------------------------#
proc ImageSelect { input_file_name input_group input_dataset input_position } \
   {
   global MISActiveServer MISActiveEntry
   global enMISDataSet enMISSensorBand enMISDate enMISBandNumber
   global MISEntries 
   global MISLBox
   global MISLat MISLon MISGlobe MISEMag MISLMag MISStation
   global globe  ;# -- need so Bindings can find in global scope
   global MISMapEnabled
   global MISSelectionChange 
   global popup_help_text # access the array of popup help values
   global LastLMag LastEMag
   global NEXRAD_ID ;# -- NEXRAD station ID number
   global RADAR_Options ;# -- ON or OFF, selection for use of predifined products
   global data
   global ServerIndex 
   global EntryIndex
   global bar
   global env
   global groupDatasetPosition
   global band_list
   global ListBoxSelections


   # Create one variable containing the group, dataset, and position
     set groupDatasetPosition $input_group/$input_dataset\.$input_position

   # Truncate input file name if it is too long
   if {[string length $input_file_name] > 30} \
   {
	set input_file_name "[string range $input_file_name 0 26]..."
   }

   # Create a list of available bands
   CREATE_BAND_LIST $groupDatasetPosition

   # --- Initialize NEXRAD ID variable to none so that it can be used as a flag.
     set NEXRAD_ID "none"

   #--- Initialize the list data correctly (use last active Server if applicable else first in list)
     if { ! [info exists MISActiveServer] } { set MISActiveServer 0 }
     if { ! [info exists MISActiveEntry] }  { set MISActiveEntry 0 }
     MISBuildList
     
   #--- Initialize some global variables 
     set enMISBandNumber 100  ;# intentionally out of range
     set enMISDate   99999999 ;# also intentionally out of range
     set MISLat ""
     set MISLon ""
     set MISLMag 1
     set MISEMag 1
     set MISMapEnabled 0
     set MISStation ""
     set MISSelectionChange 0
  
   toplevel .tlImageSelect -borderwidth 3 -relief raised
   wm title .tlImageSelect "File Properties"
   wm geometry .tlImageSelect 475x625+750+50
   wm resizable .tlImageSelect 0 0
 
   frame .tlImageSelect.fmBody
     Notebook:create .tlImageSelect.nbImageDisp -pages { Data Location } -pad 10\
        -width 475 -height 510
     pack .tlImageSelect.nbImageDisp -fill both -expand true

     #---- DATA section ---------------------------------------
     set data [Notebook:frame .tlImageSelect.nbImageDisp Data]
       #---- Menues frame
       frame $data.frameMenu
#ml         LMCreateLoopButton $data.frameMenu "  Select Frame  " YES
         #--- Dataset menu
         frame $data.frameMenu.frameDS
#ml	 pack $data.frameMenu.frameDS -side top -anchor w -fill x 
         #--- Band menu
         frame $data.frameMenu.frameBand
#ml	 pack $data.frameMenu.frameBand -side top -anchor w -fill x
	 #--- Date Range
         frame $data.frameMenu.frameDate
#ml	 pack $data.frameMenu.frameDate -side top -anchor w -fill x 

       #ml -- Directory selection frame
	 frame $data.frameMenu.frameDir \
	   -relief groove \
	   -borderwidth 2
#	 pack $data.frameMenu.frameDir \
#	   -side top \
#	   -anchor w \
#	   -fill x \
#	   -pady 10

       #--Listbox frame      	      
         frame $data.frameMenu.frameList
	 pack $data.frameMenu.frameList -side top -fill both -expand true
       pack $data.frameMenu -side top -pady 10 -padx 10 -fill both -expand true
	 
   pack .tlImageSelect.fmBody  
   
   #---Controls frame (load/dismiss/load&dismiss/help)
   frame .tlImageSelect.fmBottom 
     button .tlImageSelect.fmBottom.btLoad -text "Load Image" -command MISLoadImages
     button .tlImageSelect.fmBottom.btLoadDismiss -text "Load & Dismiss" \
	-command  { MISLoadImages; destroy .tlImageSelect}
     button .tlImageSelect.fmBottom.btDismiss -text "Dismiss" \
	-command {destroy .tlImageSelect}
     button .tlImageSelect.fmBottom.btHelp \
	-text "Help" \
	-command "execGUI helplite HELPDISPLAY"
     pack .tlImageSelect.fmBottom.btLoad -side left -padx 2 -pady 5
     pack .tlImageSelect.fmBottom.btLoadDismiss -side left -padx 2 -pady 5
     pack .tlImageSelect.fmBottom.btHelp -side right -padx 2 -pady 5
     pack .tlImageSelect.fmBottom.btDismiss -side right -padx 2 -pady 5
   pack .tlImageSelect.fmBottom -padx 10 -pady 5

   #---Initialize the RADAR options frame
     RADAR_Options INIT X X

   #--- Get data from IMGLIST to serve as the header 
     set inputOperator [open "|imglist.k $groupDatasetPosition FORM=EXP" r]
     for {set i 0} {$i < 5} {incr i} {gets $inputOperator line1}
     gets $inputOperator line2
     catch {close $inputOperator}
     
     set header       [format "%-45.45s" "Image Information:"]
     set image_file   [format "%-45.45s" "    File Name: $input_file_name"]
     set image_source [format "%-45.45s" "    Source:    [string trim [string range $line1  5 17]]"]
     set image_date   [format "%-45.45s" "    Date:      [string trim [string range $line1 20 31]]"]
     set image_time   [format "%-45.45s" "    Time:      [string trim [string range $line1 34 41]]"]
     set image_latlon [format "%-45.45s" "    Center:    [string trim [string range $line1 44 52]] (lat/lon)"]
     set image_size   [format "%-45.45s" "    Size:      [string trim [string range $line2 67 78]]"]

   #-- The List box -- need MISLBox to be defined for DataSet section
#ml     set MISLBox [MakeListBox $data.frameMenu.frameList "" " Satelite     Date     Time    Center Coordinates   Band(s)" ]
      set MISLBox [MakeListBox $data.frameMenu.frameList \
		   "$header\n\n$image_file\n$image_source\n$image_date\n$image_time\n$image_latlon\n$image_size\n" \
		   "Select a Band from the List Below:"]
     $MISLBox config -selectmode single 

   #------------ Fill the menus here ----------------------------
     set f $data.frameMenu.frameDS
     set enMISDataSet ""
     
     menubutton $f.mb -text "Dataset" -menu $f.mb.menu \
                      -relief raised -borderwidth 2 -indicatoron 1 -width 12
#ml     pack  $f.mb -side left
     entry $f.entryDS -textvar enMISDataSet -state disabled -width 26
#ml     pack  $f.entryDS -side left -fill x -expand true
     set m [menu $f.mb.menu -tearoff 0]
     set ServerIndex 0

     while { [info exists MISEntries($ServerIndex)] } \
       {
       $m add cascade -label $MISEntries($ServerIndex) -menu $m.sub$ServerIndex 
       set EntryIndex 0
       set m2 [menu $m.sub$ServerIndex -tearoff 0]

       while { [info exists MISEntries($ServerIndex,$EntryIndex)] }\
         {
         #--Add call to the NEXRAD GUI if a NEXRAD dataset is chosen.  If the User hits
	 #  'Cancel' on the NEXRAD GUI then NEXRAD_ID is set to 'canceled'.
	   if {[string compare $MISEntries($ServerIndex) "NEXRAD"] == 0} \
  	   {$m2 add command -label $MISEntries($ServerIndex,$EntryIndex) \
			    -command "RADAR_Options ADD $ServerIndex $EntryIndex"
	   } \
	   else \
	   {
           $m2 add command -label $MISEntries($ServerIndex,$EntryIndex) \
		    	   -command "set enMISSensorBand {}; \
				     set enMISDate {}; \
				     set NEXRAD_ID none; \
				     RADAR_Options REMOVE X X; \
			 	     $MISLBox selection clear 0 [expr {[$MISLBox size] - 1}]; \
			    	     MISDoList $ServerIndex $EntryIndex" 
	   }

	 incr EntryIndex
         }
       incr ServerIndex   
       }   
     pack $f -side top -fill x 
     
   #ml--Directory selection widgets
     set f $data.frameMenu.frameDir

     frame $f.entryFrame
     pack $f.entryFrame \
	-side top \
	-fill x \
	-expand true \
	-padx 10 \
	-pady 5

     label $f.entryFrame.label \
	-text "Current Directory: " \
	-font {-adobe-courier-bold-r-normal--*-90*}
     pack $f.entryFrame.label \
	-side left

     entry $f.entryFrame.currentDirectory
     pack $f.entryFrame.currentDirectory \
	-side left \
	-fill x \
	-expand true
     $f.entryFrame.currentDirectory insert end $env(MCLITE_INSTALL)/data
     $f.entryFrame.currentDirectory configure \
	-state disabled

     frame $f.buttonFrame
     pack $f.buttonFrame \
	-side top \
	-fill x \
	-expand true \
	-padx 10 \
	-pady 5

     label $f.buttonFrame.label \
	-text "Click to change to a different directory:" \
	-font {-adobe-courier-bold-r-normal--*-90*}
     pack $f.buttonFrame.label \
	-side left

     button $f.buttonFrame.button \
	-text "Change Directory" \
	-command {FileSelector}
     pack $f.buttonFrame.button \
	-side left

   #--Sensor/Band Menu	 
     set f $data.frameMenu.frameBand
     menubutton $f.buttonBand -text "Sensor/Band" \
	       -menu $f.buttonBand.menu -relief raised \
               -borderwidth 2 -highlightthickness 0 -indicatoron 1 \
	       -width 12
     pack $f.buttonBand -side left
     set m [menu $f.buttonBand.menu -tearoff 0]
     entry $f.entryBand -textvar enMISSensorBand -state disabled -width 26
     set enMISSensorBand ""  
     pack $f.entryBand -side left -fill x -expand true

   #--Date Menu
     set f $data.frameMenu.frameDate
     menubutton $f.buttonDate -text "Date" \
	       -menu $f.buttonDate.menu -relief raised \
               -borderwidth 2 -highlightthickness 0 -indicatoron 1 \
	       -width 12
     pack $f.buttonDate -side left
     set m [menu $f.buttonDate.menu -tearoff 0]
     entry $f.entryDate -textvar enMISDate -state disabled -width 26
     set enMISDate ""
     pack $f.entryDate -side left -fill x -expand true

    #--BAR checkbutton
     frame $data.frameMenu.barFrame
     pack $data.frameMenu.barFrame \
 	-side top \
	-padx 10 \
	-fill x \
	-expand true

     checkbutton $data.frameMenu.barFrame.barButton \
	-text "Display a grayscale bar on the image" \
 	-variable bar \
	-onvalue YES \
	-offvalue NO \
	-highlightthickness 0
     pack $data.frameMenu.barFrame.barButton \
	-anchor w
#     $data.frameMenu.barFrame.barButton select

    #--MAP options
      frame $data.frameMenu.fmMap -relief groove -borderwidth 3
	 checkbutton $data.frameMenu.fmMap.cbUseMap -text "Display Map On Image" -variable MISMapEnabled -highlightthickness 0 \
	          -offvalue 0 -onvalue 1
	 pack $data.frameMenu.fmMap.cbUseMap -side left -padx 5 -pady 5
         # --- insert CONFIG MAP option here
	 button $data.frameMenu.fmMap.btConfig -text "Configure Map" -command "ConfigMap NODRAW"
	 pack $data.frameMenu.fmMap.btConfig -side right -padx 5 -pady 5
#ml      pack $data.frameMenu.fmMap -side top -padx 10 -fill x -expand true

   #---- OPTIONS section ---------------------------------------
#ml     set options [Notebook:frame .tlImageSelect.nbImageDisp Options ]
#ml       #---- Menues frame
#ml       frame $options.fmOptions
#ml	 #--EU frame
#ml	   frame $options.fmOptions.frameEU 
#ml             MISSetEU $options.fmOptions.frameEU
#ml           pack $options.fmOptions.frameEU -side top -expand true -fill x 
#ml	 #--SU frame
#ml	   frame $options.fmOptions.frameSU 
#ml             MISSetSU $options.fmOptions.frameSU 
#ml           pack $options.fmOptions.frameSU -side top -expand true -fill x
#ml	 #--GrayBar frame
#ml	   frame $options.fmOptions.frameGB 
#ml             MISSetGB $options.fmOptions.frameGB 
#ml           pack $options.fmOptions.frameGB -side top -expand true -fill x
#ml	 #--Label Text
#ml	   frame $options.fmOptions.fmLabel  
#ml            label $options.fmOptions.fmLabel.lbLabel -width 16 -text "Caption Text"
#ml	     entry $options.fmOptions.fmLabel.enEntry -textvariable MISLabel \
#ml                            -bg white -borderwidth 3 -highlightthickness 0 
#ml             pack $options.fmOptions.fmLabel.lbLabel -side left
#ml	     pack $options.fmOptions.fmLabel.enEntry -side left -fill x -expand true -padx 3
#ml             set MISLabel ""
#ml           pack $options.fmOptions.fmLabel -side top -expand true -fill x
#ml       pack $options.fmOptions -side top -padx 10 -pady 10 -anchor w

   #---- Globe section ---------------------------------------
     set globe [Notebook:frame .tlImageSelect.nbImageDisp Location ]
       frame $globe.fmMain
         frame $globe.fmMain.fmGlobe
           frame $globe.fmMain.fmGlobe.fmframe -relief sunken -borderwidth 3
             rglobe $globe.fmMain.fmGlobe.fmframe.glGlobe \
		-width 360 \
		-height 360 \
		-dataset "" \
		-latitude MISLat \
		-longitude MISLon \
		-elementmag MISEMag \
		-linemag MISLMag \
		-id $NEXRAD_ID
             set MISGlobe $globe.fmMain.fmGlobe.fmframe.glGlobe		   
	     pack $globe.fmMain.fmGlobe.fmframe.glGlobe -side top
           pack $globe.fmMain.fmGlobe.fmframe -side left 
	   button $globe.fmMain.fmGlobe.btMouse \
		-bitmap @$env(MCLITE_INSTALL)/lib/gui/mouse.xbm \
		-command "ShowMouseHelp $globe.fmMain.fmGlobe.btMouse"
	   pack $globe.fmMain.fmGlobe.btMouse -side right -anchor ne -padx 5
	   set popup_help_text($globe.fmMain.fmGlobe.btMouse) "Show Mouse Help"
	 pack $globe.fmMain.fmGlobe -side top -padx 10 -pady 5  
         frame $globe.fmMain.fmEcho 
	   frame $globe.fmMain.fmEcho.fmLat
  	     label $globe.fmMain.fmEcho.fmLat.lbLat -text "Lat:" -anchor e
	     entry $globe.fmMain.fmEcho.fmLat.enLat -textvariable MISLat -width 8 -bg white
  	     pack $globe.fmMain.fmEcho.fmLat.lbLat -side left 
	     pack $globe.fmMain.fmEcho.fmLat.enLat -side left
	   pack $globe.fmMain.fmEcho.fmLat -side left 
	   frame $globe.fmMain.fmEcho.fmLon
	     label $globe.fmMain.fmEcho.fmLon.lbLon -text "Lon:" -anchor e
	     entry $globe.fmMain.fmEcho.fmLon.enLon -textvariable MISLon -width 8 -bg white
	     pack $globe.fmMain.fmEcho.fmLon.lbLon -side left
  	     pack $globe.fmMain.fmEcho.fmLon.enLon -side left
	   pack $globe.fmMain.fmEcho.fmLon -side left -padx 5

         #-- Station Location
	   frame $globe.fmMain.fmEcho.fmStation  
             label $globe.fmMain.fmEcho.fmStation.lbLabel -text "Station:" -anchor e
	     entry $globe.fmMain.fmEcho.fmStation.enEntry -textvariable MISStation \
                            -bg white -borderwidth 3 -highlightthickness 0 -width 8
 
             bind $globe.fmMain.fmEcho.fmStation.enEntry <KeyPress> { set MISStation [string toupper $MISStation] }
	     bind $globe.fmMain.fmEcho.fmStation.enEntry <KeyPress-Return> { StationToLL $globe }
             bindtags $globe.fmMain.fmEcho.fmStation.enEntry [list Entry . $globe.fmMain.fmEcho.fmStation.enEntry all ]

             pack $globe.fmMain.fmEcho.fmStation.lbLabel -side left
	     pack $globe.fmMain.fmEcho.fmStation.enEntry -side left 
           pack $globe.fmMain.fmEcho.fmStation -side top -expand true -fill x
	 pack $globe.fmMain.fmEcho -side top -padx 10 -fill x -expand true
	 
	 # --- Magnification sliders and entries
	 frame $globe.fmMain.fmMag
	   frame $globe.fmMain.fmMag.fmyMag 
	     frame $globe.fmMain.fmMag.fmyMag.fmLabel 
 	       label $globe.fmMain.fmMag.fmyMag.fmLabel.lby -text "\nLine Mag:" -width 12 -anchor e
               pack $globe.fmMain.fmMag.fmyMag.fmLabel.lby -side top 
  	     pack $globe.fmMain.fmMag.fmyMag.fmLabel -side left 
	     scale $globe.fmMain.fmMag.fmyMag.lmag -from -32 -to 32 -orient horizontal \
                  -length 300 -variable MISLMag -highlightthickness 0 \
		  -command { MISModifyLMag; set dummy }
	     pack $globe.fmMain.fmMag.fmyMag.lmag -side left -anchor n
	   pack $globe.fmMain.fmMag.fmyMag -side top 
	   frame $globe.fmMain.fmMag.fmxMag 
	     frame $globe.fmMain.fmMag.fmxMag.fmLabel 
 	       label $globe.fmMain.fmMag.fmxMag.fmLabel.lbx -text "\nElement Mag:" -width 12 -anchor e
               pack $globe.fmMain.fmMag.fmxMag.fmLabel.lbx -side top 
  	     pack $globe.fmMain.fmMag.fmxMag.fmLabel -side left 
	     scale $globe.fmMain.fmMag.fmxMag.emag -from -32 -to 32 -orient horizontal \
                  -length 300 -variable MISEMag -highlightthickness 0 \
		  -command { MISModifyEMag; set dummy }
	     pack $globe.fmMain.fmMag.fmxMag.emag -side left -anchor n
	   pack $globe.fmMain.fmMag.fmxMag -side top 
	 pack $globe.fmMain.fmMag -side top -padx 10 -fill x -expand true
       pack $globe.fmMain -side top

    #--- need to set up some bindings in case MISEMag changes
      bind $globe.fmMain.fmEcho.fmLon.enLon <FocusOut> \
	{ $globe.fmMain.fmGlobe.fmframe.glGlobe setlon $MISLon }

      bind $globe.fmMain.fmEcho.fmLat.enLat <FocusOut> \
	{ $globe.fmMain.fmGlobe.fmframe.glGlobe setlat $MISLat }

      bind $globe.fmMain.fmEcho.fmLon.enLon <KeyPress-Return> \
	{ set MISStation ""
	  $globe.fmMain.fmGlobe.fmframe.glGlobe setlon $MISLon }

      bind $globe.fmMain.fmEcho.fmLat.enLat <KeyPress-Return> \
	{ set MISStation ""
	  $globe.fmMain.fmGlobe.fmframe.glGlobe setlat $MISLat }

      bind $globe.fmMain.fmEcho.fmStation.enEntry <KeyPress-Return> \
	{ StationToLL $globe
	   $globe.fmMain.fmGlobe.fmframe.glGlobe setlon $MISLon
	   $globe.fmMain.fmGlobe.fmframe.glGlobe setlat $MISLat }

      bind $globe.fmMain.fmEcho.fmStation.enEntry <FocusOut> \
	{ StationToLL $globe
	  $globe.fmMain.fmGlobe.fmframe.glGlobe setlon $MISLon
	  $globe.fmMain.fmGlobe.fmframe.glGlobe setlat $MISLat } 

      # Actions to be taken when the 'Location' tab is selected
      bind $globe.fmMain.fmGlobe.fmframe.glGlobe  <Visibility> \
      { 
	# If a new image has been selected, initialize globe 
	if { $MISSelectionChange } \
	{ 
		set MISSelectionChange 0
		MISClick

		# If a new NEXRAD image has been selected, set defaults
      		if {[string compare $NEXRAD_ID "none"] != 0} \
      		{
			set MISStation $NEXRAD_ID
			StationToLL $globe
			$globe.fmMain.fmGlobe.fmframe.glGlobe setlon $MISLon
			$globe.fmMain.fmGlobe.fmframe.glGlobe setlat $MISLat
     		} \
		else \
		{
			set MISStation ""
			StationToLL $globe
		}
	} 
      }  

      bind $globe.fmMain.fmGlobe.fmframe.glGlobe  <Button-1> { set MISStation "" }  
      bind $globe.fmMain.fmGlobe.fmframe.glGlobe  <Button-2> { set MISStation "" }

    #--- set a binding to the list box to alter MISSelection so that the globe is updated
      bind $MISLBox <Button-1> { set MISSelectionChange 1; \
				 set MISEMag 1; \
				 MISModifyEMag; \
				 set MISLMag 1; \
				 MISModifyLMag }
      

    # -- Place the list of bands into the list box
    FILL_LIST_BOX

#ml    MISClick

    #ml -- Call MISDoList with the appropriate parameters depending
    #	   upon which dataset is needed (reads SERVERTABLE.TDAT)
#ml      if {[string compare $input_dataset "MCLITE1"] == 0} \
#ml      {
#ml	MISDoList 0 0
#ml      } \
#ml      elseif {[string compare $input_dataset "MCLITE2"] == 0} \
#ml      {
#ml	# MODIS servers necessary for this to work
#ml	MISDoList 0 1
#ml      } \
#ml      else \
#ml      {
	update
	MISBuildBandList
    	$MISLBox selection set 0
    	MISClick
#ml      }

   set ListBoxSelections($MISLBox) 0
   return
   }

#--------------------------------------------------------------------
     proc MISModifyLMag { } \
       {		 
       #--- routine to interact with SCALE widget for setting Line Magnification 
       global MISLMag LastLMag MISEMag MISGlobe
 
       #--- don't allow Mag of 0 or -1 so trap this and set to 1 or -2 based on 
       #--- the last time that it was set
       if { $MISLMag == 0 } \
       { 
	 if { $LastLMag >= 0 } \
	 {
		set MISLMag -1
	 } \
	 else \
	 {
		set MISLMag 1
	 } 
       }

       set MISEMag $MISLMag
       $MISGlobe setlmag $MISLMag  ;# Notify globe of change
       $MISGlobe setemag $MISLMag  ;# record the last value
       set LastLMag $MISLMag

       }

#--------------------------------------------------------------------------------------
     proc MISModifyEMag { } \
     {		  
       #--- routine to interact with SCALE widget for setting Element Magnification 
       global MISLMag LastEMag MISEMag MISGlobe

       #--- don't allow Mag of 0 or -1 so trap this and set to 1 or -2 based on 
       #--- the last time that it was set
       if { $MISEMag == 0 } \
       { 
	 if { $LastEMag >= 0 } \
	 { 
		set MISEMag -1
	 } \
	 else \
	 {
		set MISEMag 1
	 } 
       }

       $MISGlobe setemag $MISEMag ;# Notify globe of change
       set LastEMag $MISEMag      ;# record the last value

     }
   
   
   
#-----------------------------------------------------------------------------------#
# ShowMouseHelp -- brief popup window to show mouse button functions                #
#-----------------------------------------------------------------------------------#
proc ShowMouseHelp { widget } \
  {
  global popup_help_option      # from popuphelp.tcl
  global MISGlobe

  # get current position of mouse in widget
    set mx [winfo pointerx $widget]
    set my [winfo pointery $widget]

  # make sure the mouse is in this widget
    set has_mouse [winfo containing $mx $my]
    if {$has_mouse != $widget } { return }

  # if the popup_help window doesn't exist, create it
    if ![winfo exists .mouse_help] {
      toplevel .mouse_help -relief raised -bd 2
      wm override .mouse_help 1
      set check  [$MISGlobe cget -Nav] 
      if { [string match $check "POLAR"] } \
      {
	set tlab "Left Button -- No function"
      } \
      else { set tlab "Left Button -- Drag Selection Bounds" }
      label .mouse_help.text1 -bd 0 -highlightthickness 0 \
	-bg $popup_help_option(background) \
	-fg $popup_help_option(foreground) \
	-text $tlab -anchor w
      pack .mouse_help.text1 -side top -anchor w -fill x -expand true
      label .mouse_help.text2 -bd 0 -highlightthickness 0 \
	-bg $popup_help_option(background) \
	-fg $popup_help_option(foreground) \
	-text "Middle Button -- Set Lat/Lon Center point" -anchor w
      pack .mouse_help.text2 -side top -anchor w -fill x -expand true
      label .mouse_help.text3 -bd 0 -highlightthickness 0 \
	-bg $popup_help_option(background) \
	-fg $popup_help_option(foreground) \
	-text "Right Button -- No function" -anchor w
      pack .mouse_help.text3 -side top -anchor w -fill x -expand true
    }
  # Position window a little bit down and to right of widget.
  #
  set x [winfo rootx $widget]
  set y [expr [winfo rooty $widget] +10] ; # + [winfo height $widget]]
#  set w [winfo width .mouse_help]
#  set W [expr [winfo rootx .] + [winfo width .]]
#  if { [expr $x + $w] > $W } {
#    set x [expr $W - $w + 10]
#  }

  set x [expr $x-200]


  wm geometry .mouse_help "+$x+$y"

  # Show window.
  #
  wm deiconify .mouse_help
  raise .mouse_help
  
  bind .mouse_help <Leave> { destroy .mouse_help }

  }   

#-----------------------------------------------------------------------------------#
# StationToLL { globe } -- convert station name to Lat Lon coordinates              #
#-----------------------------------------------------------------------------------#   
proc StationToLL { globe } \
  {
  global MISStation
  global MISLat MISLon
  global MISLMag MISEmag


# --- Exit if no station has been specified
  set MISStation [string trim $MISStation]
  if {[string compare $MISStation ""] == 0} {return}

# --- Convert any input to upper case
  set MISStation [string toupper $MISStation]

# --- Create input operator for call to STNLIST to get lat/lon 
#     coordinates for the given station
  set StationLookup [open "|stnlist.k $MISStation" r]
  gets $StationLookup line1

# --- Return if there are no stations
  if {[string compare [lindex $line1 0] "No"] == 0} {return}

# --- Read station info here
  gets $StationLookup line2
  while { [gets $StationLookup line1] !=-1 } { }
  catch { close $StationLookup }

# --- Get Lat/Lon coordinates of station  
  set Lat [string range $line2 57 65]
  set Lon [string range $line2 67 76]

# --- Separate degrees, minutes, and seconds from Lat
   set seconds [string range $Lat 7 8]
   set minutes [string range $Lat 4 5]
   set degrees [string range $Lat 0 2]

# --- Remove leading zeros if necessary
   if {[string compare [string index $minutes 0] "0"] == 0} \
   {
	set minutes [string index $minutes 1]
   }

   if {[string compare [string index $seconds 0] "0"] == 0} \
   {
	set seconds [string index $seconds 1]
   }

# --- Convert Lat from DMS to decimal
   set seconds [expr {double($seconds) / 3600.0}]
   set minutes [expr {double($minutes) / 60.0}]
   set MISLat [expr {double($degrees + $minutes + $seconds)}]

# --- Truncate all but the first two decimal points in MISLat
   set dot [string first "." $MISLat]
   set MISLat [string range $MISLat 0 [expr {$dot + 2}]]

# --- Separate degrees, minutes, and seconds from Lon
   set seconds [string range $Lon 8 9]
   set minutes [string range $Lon 5 6]
   set degrees [string range $Lon 0 3]

# --- Remove leading zeros if necessary
   if {[string compare [string index $minutes 0] "0"] == 0} \
   {
	set minutes [string index $minutes 1]
   }

   if {[string compare [string index $seconds 0] "0"] == 0} \
   {
	set seconds [string index $seconds 1]
   }

# --- Convert Lon from DMS to decimal
   set seconds [expr {double($seconds) / 3600.0}]
   set minutes [expr {double($minutes) / 60.0}]
   set MISLon [expr {double($degrees + $minutes + $seconds)}]
   set MISLon [string range $MISLon 0 6]

# --- Truncate all but the last two decimal points in MISLon
   set dot [string first "." $MISLon]
   set MISLon [string range $MISLon 0 [expr {$dot + 2}]]

  }
   
#-----------------------------------------------------------------------------------#   
# proc MISBuildList { } -- Fills in the MISEntries array		            #
#-----------------------------------------------------------------------------------#
proc MISBuildList { } \
   {
   global MISEntries 
   global ServerList ServerListLength
   global groupDatasetPosition
   

   #--Clear the old settings
   catch { unset MISEntries }
   #--set up correct access to the arrays
   set ServerIndex 0
   
   for { set i 0 } { $i < $ServerListLength } { incr i } \
     {
     if { [string match "I" [lindex $ServerList($i-H) 2] ] } \
       {
       # Have a valid IMAGE SERVER so add to the list
       set MISEntries($ServerIndex) [lindex $ServerList($i-H) 0]
       set descriptors [lindex $ServerList($i-H) 8] 
       set EntryIndex 0
       for { set j 0 } { $j<$descriptors } { incr j } \
         {
	 set line $ServerList($i-$j)
	 if { [string match "I" [lindex $line 0 ] ] } \
	   {
           set MISEntries($ServerIndex,$EntryIndex) [format "%-10s %-20s" [lindex $line 1] [lrange $line 2 end]]
	   incr EntryIndex
	   }
  	 }
       incr ServerIndex	 
       }
     }
   }


#-----------------------------------------------------------------------------------------#
# proc MISDoList { ServerIndex EntryIndex  } -- get a detailed Image list at this location#
#	-- ServerIndex -- server to use							  #
#	-- EntryIndex  -- DataSet in the Server to use					  #
#-----------------------------------------------------------------------------------------#
proc MISDoList { ServerIndex EntryIndex } \
   {
    global MISEntries 
    global enMISDataSet
    global MISActiveServer MISActiveEntry
    global NEXRAD_ID
    global MISLBox
    global enMISSensorBand
    global station_exists
    global groupDatasetPosition


  # -- set the label header here
#ml    set enMISDataSet "WORKING" 
    update
     
  # -- set the Server and Entry correctly
#ml    set MISActiveServer $ServerIndex
#ml    set MISActiveEntry $EntryIndex

  # -- set the label header here
#ml     set Group $MISEntries($MISActiveServer)

#ml     set enMISDataSet "$Group -- [string trimleft $MISEntries($MISActiveServer,$MISActiveEntry)]" 
    
#ml  # -- If NEXRAD station ID is invalid clear image list and return
#ml	if {([string compare $NEXRAD_ID "none"] != 0) && \
#ml            ([string compare $station_exists "false"] == 0)} \
#ml	{
#ml		$MISLBox del 0 end
#ml		set enMISSensorBand "Empty Dataset - station $NEXRAD_ID does not exist"
#ml		return
#ml	}

  # -- Build new image list
#ml    MISBuildImageList

  # -- OK apply our Filter now
#ml    MISBuildBandMenu	
#ml    MISBuildDateMenu

  #ml -- Creates list of available bands with corresponding information
    MISBuildBandList

#ml Set the initial list box selection to the first item so that if the
#ml 'load' button is pressed or the 'Location' tab is used there will
#ml be a reasonable responce.
    $MISLBox selection set 0
    MISClick
  	      
   }


#-------------------------------------------------------------------------------#
# MISBuildBandList { } -- Fills the list box with a list of the available
#			  bands for the image that has been selected 
#			  (this routine has been added for McIDAS-Lite)
#-------------------------------------------------------------------------------#
proc MISBuildBandList { } \
{
   global MISEntries MISActiveServer MISActiveEntry NEXRAD_ID MISLBox
   global input_position
   global groupDatasetPosition
   global MISLBox
   global MISFilteredList

   #--Change cursor to watch
   showBusyStatus

   MISApplyFilter

   #--- first get the GROUP and DESCRIPTOR
#ml      set Group $MISEntries($MISActiveServer)
#ml      set Descriptor [lindex $MISEntries($MISActiveServer,$MISActiveEntry) 0]

   #--- build band list and display in list box
      set dot "."
      set inputOperator [open "|imglist.k $groupDatasetPosition FORM=EXP" r]

      $MISLBox delete 0 end

      set counter 0
      while {[gets $inputOperator line] > -1} \
      {
	if {[string compare [lindex $line 0] "Band:"] == 0} \
	{
	   set MISEntries($MISActiveServer,$MISActiveEntry,$counter,band_line) $line
	   $MISLBox insert end [string range $line 0 52]
	   set MISFilteredList($counter) "$MISActiveServer,$MISActiveEntry,[expr {$counter + 1}]"
	   incr counter
	}
      }

      catch {close $inputOperator}

   #--Reset cursor to normal
   showIdleStatus

}

#-------------------------------------------------------------------------------#
# MISBuildImageList { } -- Fills the MISEntries "data structures" with data     #
# 			   pointed to by the MISActiveServer and MISActiveEntry #
#-------------------------------------------------------------------------------#
proc MISBuildImageList { } \
   {
   global MISEntries MISActiveServer MISActiveEntry NEXRAD_ID MISLBox
   
#ml   ProgressInit "Building Image List"
      
   #--- first get the GROUP and DESCRIPTOR
      set Group $MISEntries($MISActiveServer)
      set Descriptor [lindex $MISEntries($MISActiveServer,$MISActiveEntry) 0]

   # --- Add ID keyword to call to IMGLIST if NEXRAD data is being used.
      if {[string compare $NEXRAD_ID "none"] != 0} \
	 {
		set DescriptorList [open "|imglist.k $Group/$Descriptor.-9999 ID=$NEXRAD_ID" r]
      	 } \
      else \
	 {
		set DescriptorList [open "|imglist.k $Group/$Descriptor.ALL" r]
	 }
      
   #--- burn the first four lines of text and verify logged on as well   
      gets $DescriptorList line	; # burn 1
      gets $DescriptorList line	; # burn 2
      if { [string first "Accounting data was not valid" $line] != -1 }  \
         {
         tk_messageBox -title "Server Error" -message "You are not logged on. \nPlease log on and try again"\
         		-type ok -parent . -icon warning
#ml         ProgressClose			
	 catch { close $DescriptorList }
         return		
         }
         
      gets $DescriptorList line	;# burn 3

   #--- OK now burn the 4th line and proceed with reading the data  -- 4th line will not exist in empty dataset
      if { [gets $DescriptorList line] == -1 } \
         {
         catch { close $DescriptorList }

         # -- Clear the list box
         $MISLBox delete 0 end

#ml	 ProgressClose
         return
         }
      
      set ImageIndex 0
      while { 1 } \
         {
	 #--- read the line and exit if done
         if { [gets $DescriptorList line] == -1 } \
           {
           catch { close $DescriptorList }
#ml	   ProgressClose
           return
           }
         # --- go until this line is read and then close and return
         if { [string match "imglist.k: done" $line] } \
            {
            catch { close $DescriptorList }
#ml	    ProgressClose
            return
            }
         # --- OK next line is the Position --  Sensor -- Date -- Time -- Lat -- Lon -- Band information
         #		              0-3        5-17    20-31    34-41	 44-47 49-52   54-end
	 set MISEntries($MISActiveServer,$MISActiveEntry,$ImageIndex,P) [lindex $line 0]		; # Position
	 set MISEntries($MISActiveServer,$MISActiveEntry,$ImageIndex,S) [string range $line 5 17]	; # Sensor
	 set MISEntries($MISActiveServer,$MISActiveEntry,$ImageIndex,DT) [format "%s %s" \
	         [string range $line 20 31] [string range $line 34 41 ] ]			; # Date and Time
	 set MISEntries($MISActiveServer,$MISActiveEntry,$ImageIndex,LL) [format "%s %s" \
	         [string range $line 44 47] [string range $line 49 52 ] ]			; # Lat and Lon
         set MISEntries($MISActiveServer,$MISActiveEntry,$ImageIndex,B) \
	         [MISBandBits [string range $line 54 end]]					; # Band Bits (flags)

#puts "\nMISEntries($MISActiveServer,$MISActiveEntry,$ImageIndex,P)=$MISEntries($MISActiveServer,$MISActiveEntry,$ImageIndex,P)"
#puts "MISEntries($MISActiveServer,$MISActiveEntry,$ImageIndex,S)=$MISEntries($MISActiveServer,$MISActiveEntry,$ImageIndex,S)"
#puts "MISEntries($MISActiveServer,$MISActiveEntry,$ImageIndex,DT)=$MISEntries($MISActiveServer,$MISActiveEntry,$ImageIndex,DT)"
#puts "MISEntries($MISActiveServer,$MISActiveEntry,$ImageIndex,LL)=$MISEntries($MISActiveServer,$MISActiveEntry,$ImageIndex,LL)"
#puts "MISEntries($MISActiveServer,$MISActiveEntry,$ImageIndex,B)=$MISEntries($MISActiveServer,$MISActiveEntry,$ImageIndex,B)"

         incr ImageIndex
#ml	 ProgressUpdate

         }      
   }

#---------------------------------------------------------------------------#
# proc MISClick -- when the mouse is clicked set the DATASET in the globe   #
#               -- accordingly                                              #
#---------------------------------------------------------------------------#
proc MISClick { } \
{
  global MISLBox MISGlobe 
  global MISFilteredList
  global MISEntries
  global MISLMag MISEMag
  global enMISBandNumber  
  global NEXRAD_ID		; # NEXRAD station ID
  global ListBoxSelections	; # List Box Selection Indexes saved for when LB loses focus
  global groupDatasetPosition
  global band_list
  
  showBusyStatus

  #--- get list of selection indexes
#ml    set FilterIndex [lindex [$MISLBox curselection] 0]
#ml     set FilterIndex [lindex $ListBoxSelections($MISLBox) 0]

  #--- if none then return  
#ml    if { ! [llength $FilterIndex] } { return }
  
#ml  set ServerIndex [lindex [split $MISFilteredList($FilterIndex) ,] 0]
#ml  set EntryIndex [lindex [split $MISFilteredList($FilterIndex) ,] 1]
#ml  set Group $MISEntries($ServerIndex)
#ml  set Descriptor [lindex $MISEntries($ServerIndex,$EntryIndex) 0]
#ml  set Position $MISEntries($MISFilteredList($FilterIndex),P) 

#ml     set slash [string first "/" $groupDatasetPosition]
#ml     set dot [string first "." $groupDatasetPosition]
#ml     set Group [string range $groupDatasetPosition 0 [expr {$slash - 1}]]
#ml     set Descriptor [string range $groupDatasetPosition [expr {$slash + 1}] [expr {$dot - 1}]]
#ml     set Position [string range $groupDatasetPosition [expr {$dot + 1}] end]

  #---- OK can config the dataset now
	$MISGlobe config -dataset "$groupDatasetPosition" \
			 -band 1 \
			 -id $NEXRAD_ID

  #---- Set magnification factors to one
  set MISEMag 1
  MISModifyEMag
  set MISLMag 1
  MISModifyLMag

  showIdleStatus
}


#---------------------------------------------------------------------------#
# proc MISLoadImages -- 						    #
#---------------------------------------------------------------------------#
proc MISLoadImages { } \
   {
   global MISEntries MISFilteredList MISLBox
   global enMISBandNumber
   global MISEUEcho MISSUEcho MISGBEcho MISLabel 
   global MISStation
   global MISLat MISLon
   global MISEMag MISLMag 
   global MISMapEnabled
   global MISSelectionChange
   global NEXRAD_ID
   global globe
   global RADAR_Options
   global ListBoxSelections	; # track what's been selected for when LB loses focus
   global bar
   global groupDatasetPosition


   #--Get current selection in band list box
   set current_band_selection $ListBoxSelections($MISLBox)
#puts "current_band_selection=$current_band_selection"

   #--set up lat/lon and mag if necessary
   if {[string compare $NEXRAD_ID "none"] != 0 && $MISSelectionChange} \
      {
	set MISStation $NEXRAD_ID
	StationToLL $globe
      } \
   elseif { [string length $MISLon]==0 || $MISSelectionChange } \
      {
     	MISClick  ; #simulate a click in order to initialize the globe correctly
      }

   #--build up the options here out of MISEUEcho ... MISLON
   set LoadOptions "LATLON=$MISLat $MISLon"

#ml   if { ![string match  "None" $MISEUEcho] } { append LoadOptions " EU=$MISEUEcho" }
#ml   if { ![string match  "None" $MISSUEcho] } { append LoadOptions " SU=$MISSUEcho" }
#ml   if { [string length $MISLabel] } { append LoadOptions " TEXT='  $MISLabel'" }
   
   #-- the grayscale bar
#ml   if { [string match "Default" $MISGBEcho] } { append LoadOptions " GRAY=YES" }
#ml   if { [string match "Reverse" $MISGBEcho] } { append LoadOptions " GRAY=X -2 255" }
  
   #-- line and element magnification
   append LoadOptions " MAG=$MISLMag $MISEMag" 
  
   # --- time for the watch
     showBusyStatus
   
   # --- go ahead and load the images 
   #set MarkedList [$MISLBox curselection]
   set MarkedList $ListBoxSelections($MISLBox);  # This contains the indeces of the selected 
						    # bands in the list box.

#ml   set count [expr {[llength $MarkedList] + [UCpeek 51] - 1}]
   set count 1; #will have to be changed to display multiple images

   foreach FilterIndex $MarkedList \
     {
     # -- need to break up the index to get the GROUP --> SERVER
#ml     set ServerIndex [lindex [split $MISFilteredList($FilterIndex) ,] 0]
#ml     set EntryIndex [lindex [split $MISFilteredList($FilterIndex) ,] 1]
#ml     set Group $MISEntries($ServerIndex)
#ml     set Descriptor [lindex $MISEntries($ServerIndex,$EntryIndex) 0]
#ml     set Position $MISEntries($MISFilteredList($FilterIndex),P)

     #ml-- We will derive the above information from 'groupDatasetPosition'
     set slash [string first "/" $groupDatasetPosition]
     set dot [string first "." $groupDatasetPosition]
     set Group [string range $groupDatasetPosition 0 [expr {$slash - 1}]]
     set Descriptor [string range $groupDatasetPosition [expr {$slash + 1}] [expr {$dot - 1}]]
     set Position [string range $groupDatasetPosition [expr {$dot + 1}] end]

#ml puts "\nset Position $MISEntries($MISFilteredList($FilterIndex),P)"
#ml puts "MISFilteredList($FilterIndex)=$MISFilteredList($FilterIndex)"
#ml puts "FilterIndex=$FilterIndex"
 
     #--- The frame object is now only used to keep the FrameInfolist updated
     #--- as well as updating all of the currently active ShowFrame Buttons

     # -- Add ID= keyword if NEXRAD data is to be displayed
     if {[string compare $NEXRAD_ID "none"] != 0} {append LoadOptions " ID=$NEXRAD_ID"}

     # --- Use predefined products from IMGDISP.CORE if selected by the User
     if {[string compare $RADAR_Options "ON"] == 0} \
     {
	set Group ""
	set Descriptor "MKX_$Descriptor"
	mcidas exec "EG"
     }

     #ml Get band number from list box
     set band [lindex [$MISLBox get $current_band_selection] 1]
#     puts "band=$band"

#ml     LMLoadImageIntoFrame $Group $Descriptor $Position $enMISBandNumber \
#ml			  $LoadOptions $count $MISMapEnabled $bar"

#puts "LMLoadImageIntoFrame $Group/$Descriptor\.$Position $band \
#			  $LoadOptions $count $MISMapEnabled $bar"

     LMLoadImageIntoFrame $Group $Descriptor $Position $band \
			  $LoadOptions $count $MISMapEnabled $bar

     incr count -1
     }

   # --- restore cursor
     showIdleStatus      
   }   


#-------------------------------------------------------------------------------#
# proc MISApplyFilter {  } -- builds the MISFilteredList based on MISdate and MISoffset settings
#			   -- A filtered list makes ListBox interraction much easier
#-------------------------------------------------------------------------------#
proc MISApplyFilter  { } \
   {
   global MISEntries MISFilteredList 
   global MISActiveServer MISActiveEntry
   global enMISSensorName enMISBandNumber MISjDate
   global MISLBox
   global NEXRAD_ID

   # --- if enMISSensorName is undefined then just return
      if { ![info exists enMISSensorName] } { return }
   # --- else we need the BandMask to apply
     set BandMask [expr 1<<($enMISBandNumber-1)]   

   # --- get active line so can reset there if possible
     set LBLine [$MISLBox nearest 0]

   # --- if all inclusive set IncludeAll to 1 else 0
      if {$MISjDate == -1} { set IncludeAll 1} \
      else\
         {   
         # --- set the date
         set date $MISjDate
         set IncludeAll 0
         }   
         
   # --- Need to clear all of the old entries
      if { [info exists MISFilteredList] } { unset MISFilteredList }
   
   # --- Now do the looping and insert any that meet the conditions into the list
   set ImageIndex 0
   set FilterIndex 0
   
   while { [info exists MISEntries($MISActiveServer,$MISActiveEntry,$ImageIndex,P)] } \
      {
#ml -- Disabled filter so that all images in directory will be displayed initially
#ml      if { ($BandMask & $MISEntries($MISActiveServer,$MISActiveEntry,$ImageIndex,B)) && \
#ml           ([string match $enMISSensorName $MISEntries($MISActiveServer,$MISActiveEntry,$ImageIndex,S)]) } \
#ml         {
#ml         set cdate [lindex $MISEntries($MISActiveServer,$MISActiveEntry,$ImageIndex,DT) 2]
#ml         if { $IncludeAll || ($cdate == $date) } \
#ml            {
            set MISFilteredList($FilterIndex) "$MISActiveServer,$MISActiveEntry,$ImageIndex"
            incr FilterIndex
#ml            } 
#ml        }  
      incr ImageIndex
      }
  # --  Now go ahead and insert these items into the list
     $MISLBox delete 0 end
        
     set FilterIndex 0
     while { [info exists MISFilteredList($FilterIndex)] } \
        {
	# --- sensor name -- date and time -- Lat -- Lon
	set index $MISFilteredList($FilterIndex)

  	   # --- Display NEXRAD station ID as center point location in list box
	   #     if NEXRAD data is being used, otherwise display lat/lon coordinates.
	   if {[string compare $NEXRAD_ID "none"] != 0} \
	   {
	   set text [format "%22s %10s" $MISEntries($index,DT) $NEXRAD_ID]
	   } \
	   else \
	   {
	   set text [format "%12s %22s    Lat:%4s Lon:%4s" \
		    	$MISEntries($index,S) \
	              	$MISEntries($index,DT) \
		      	[lindex $MISEntries($index,LL) 0] \
	    	      	[lindex $MISEntries($index,LL) 1] ]
	   }

        $MISLBox insert $FilterIndex $text
        incr FilterIndex
        }
   # -- now reset to position
      $MISLBox yview $LBLine        
   }


#-----------------------------------------------------------------------#
# MISBuildBandMenu { } -- inserts the SENSOR -- Band menu into the BAND button
#-----------------------------------------------------------------------# 
proc MISBuildBandMenu { } \
   {
   global MISEntries 
   global enMISSensorBand enMISBandNumber enMISSensorName
   global MISActiveServer MISActiveEntry  
   global MISSelectionChange 
   global MISLBox
     

   #   Sensors -- list of Sensor names
   #   SensorFlags -- indexed by SensorName -- flags indicating valid Sensor bands

   # --- step one build up the Sensors list
      set ImageIndex 0 
      set SensorIndex 0
      while { [info exists MISEntries($MISActiveServer,$MISActiveEntry,$ImageIndex,S)] }\
        {
        set sensor $MISEntries($MISActiveServer,$MISActiveEntry,$ImageIndex,S)
        if { ![info exists SensorFlags($sensor)] } \
          {
          set Sensors($SensorIndex) $sensor
	  set SensorFlags($sensor) 0
          incr SensorIndex
          }
        set SensorFlags($sensor) [expr $SensorFlags($sensor) | $MISEntries($MISActiveServer,$MISActiveEntry,$ImageIndex,B)]
        incr ImageIndex
        }

   # -- set up access to the Band Menu button and destroy any pre-existing menu
     set data [Notebook:frame .tlImageSelect.nbImageDisp Data]
     set Parent $data.frameMenu.frameBand.buttonBand
     catch { destroy $Parent.menu }

   # -- exit early if no Sensors around -- empty dataset
   if { ! [info exists Sensors(0)] } \
      { 
      set m [menu $Parent.menu -tearoff 0]
      set enMISSensorBand "Empty Dataset"

      # -- Clear the list box
#ml      $MISLBox delete 0 end

      return  
      }
   
   # -- go through the Sensors list first then access the MISBands from there
   set SensorIndex 0
   
   set m [menu $Parent.menu -tearoff 0]
   while { [info exists Sensors($SensorIndex)] } \
      {
      set sensor $Sensors($SensorIndex)
      $m add cascade -label $sensor -menu $m.sub$SensorIndex 
      set m2 [menu $m.sub$SensorIndex -tearoff 0]
      set bits $SensorFlags($sensor)
      for { set index 1 } { $index < 33 } { incr index} \
         {
         if { $bits & (1<<($index-1)) } \
            {
            $m2 add command -label $index -command "set enMISSensorName {$sensor} 
         			                    set enMISBandNumber $index    
         			                    MISBuildDateMenu                
						    set MISSelectionChange 1
         			                    set enMISSensorBand {$sensor -- Band $index}"
            }
         }
      incr SensorIndex     
      }
   # -- now initialize the Band filter to something   
   set enMISSensorName $Sensors(0)
   for { set index 1 } { $index < 33 } { incr index } \
      {
      if { $SensorFlags($enMISSensorName) & (1<<($index-1)) } \
         {
         set enMISSensorBand "$enMISSensorName Band $index"
         set enMISBandNumber $index   
         MISBuildDateMenu
         return
         }
      }
   }


#-------------------------------------------------------------------------------------------#
# MISBandBits { BStr } -- parses the BStr and builds an integer based on valid band numbers
#	-- 1-3,5 ==> 10111 (binary) ==> 23 (int) to check for the presence of the band
#	-- if((1<<(band-1))&bint) { have a band} else { not present }
#-------------------------------------------------------------------------------------------#
proc MISBandBits { BStr } \
   {
   # -- string is in following format (e)lement	e,e,e,e etc.. where e is either a number or a range # or #-#
   # -- so pick off up to four elements
   scan $BStr {%[^,]%[,]%[^,]%[,]%[^,]%[,]%[^,]} b(1) junk b(2) junk2 b(3) junk3 b(4)

   set RetValue 0
   set index 1
   while { [info exists b($index)] }\
      {
      # -- ok now form of either # or #-#
      if { [scan $b($index) {%[^-]%[-]%[^-]} n1 n2 n3] > 1 } \
         {
         # -- have a range since will have filled all three
         for { set i  $n1 } { $i <= $n3 } { incr i }  { set RetValue [expr $RetValue | (1<<($i-1))] }
         }\
      else\
         {
         # -- just a single band to add
         set RetValue [expr $RetValue | ( 1<<($n1-1))]
         }   
      incr index   
      }
   return $RetValue   
   }


#-----------------------------------------------------------------#
# MISBuildDateMenu -- builds a menu out of all of the valid dates #
#-----------------------------------------------------------------#
proc MISBuildDateMenu { } \
  {
  global MISEntries
  global MISActiveServer MISActiveEntry
  global enMISSensorBand enMISBandNumber enMISSensorName enMISDate
  global MISjDate
  
  #--DateRange menu
  #--Only Valid after the BAND is set
    set BandMask [expr 1<<($enMISBandNumber-1)]   
    set ImageIndex 0 
    set DateIndex 0
    while { [info exists MISEntries($MISActiveServer,$MISActiveEntry,$ImageIndex,S)] }\
      {
      set sensor $MISEntries($MISActiveServer,$MISActiveEntry,$ImageIndex,S)
      set band $MISEntries($MISActiveServer,$MISActiveEntry,$ImageIndex,B)
      set date [lindex $MISEntries($MISActiveServer,$MISActiveEntry,$ImageIndex,DT) 2]
      if { [string match $sensor $enMISSensorName]  && \
           $band & $BandMask  } \
        {
        if { ![info exists DateNames($date)] } \
	  {
	  set DateNames($date) [getnicedate $date 0]
	  set DateList($DateIndex) $date
	  incr DateIndex
	  }
        }
      incr ImageIndex
      }	

   # -- set up access to the Band Menu button and destroy any pre-existing menu
     set data [Notebook:frame .tlImageSelect.nbImageDisp Data]
     set Parent $data.frameMenu.frameDate.buttonDate
     catch { destroy $Parent.menu }

   # -- exit early if no Dates around -- empty dataset
   if { ! [info exists DateList(0)] } \
      { 
      set m [menu $Parent.menu -tearoff 0]
      set enMISDate ""
      return  
      }
   
   # -- go through the Sensors list first then access the MISBands from there
   set DateIndex 0
   
   set m [menu $Parent.menu -tearoff 0]
   while { [info exists DateList($DateIndex)] } \
     {
     set date $DateNames($DateList($DateIndex))
     $m add command -label $date -command " set enMISDate {$date}
                                             set MISjDate $DateList($DateIndex)
                                             MISApplyFilter "
     incr DateIndex     
     }
   $m add command -label "ALL AVAILABLE" -command "set enMISDate {ALL AVAILABLE}; set MISjDate -1; MISApplyFilter"
      
   # -- now initialize the Date filter to something   
   set MISjDate $DateList(0)
   set enMISDate [getnicedate $MISjDate 0]
   MISApplyFilter
   }   


#----------------------------------------------------------------#
# MISSetEU frm -- build the menu button and display for EU command
#----------------------------------------------------------------#
proc MISSetEU { frm } \
  {
  global MISEUEcho
  
  menubutton $frm.mbEU -text "Enhancement" -menu $frm.mbEU.menu \
                                 -relief raised -borderwidth 2 -indicatoron 1 -width 12
  pack  $frm.mbEU -side left
  entry $frm.entryEU -textvar MISEUEcho -state disabled -width 15
  pack  $frm.entryEU -side left -fill x -expand true
  set MISEUEcho "None"
  set fontset {-adobe-courier-bold-r-normal--*-90*}
  set m [menu $frm.mbEU.menu -tearoff 0]
  set EUList [open "|eu.k LIST" r]   
  # -- eat the first two lines
  gets $EUList line
  # -- this is the format line -- need it's length to accurately know the range of characters
  #    to pull for the FILENAME.ET section
    gets $EUList line
    set length [string length [lindex $line 3]]

  while { [gets $EUList line] > -1 } \
    {
    # don't use EU: DONE
    if { [string first "eu.k: Done" $line] == -1 }\
      {
      # don't use line # bytes in # files
      if { [string first "bytes in" $line] == -1 }\
        {
	set ToSet [string range $line 28 [expr 28+$length-1]]
        $m add command -label $ToSet -command "set MISEUEcho $ToSet" -font $fontset
	}
      }
    }  
  $m add command -label "None" -command "set MISEUEcho None" -font $fontset
  catch { close  $EUList }
  }


#----------------------------------------------------------------#
# MISSetSU frm -- build the menu button and display for SU command
#----------------------------------------------------------------#
proc MISSetSU { frm } \
  {
  global MISSUEcho
  
  set fontset {-adobe-courier-bold-r-normal--*-90*}

  menubutton $frm.mbSU -text "Stretching" -menu $frm.mbSU.menu \
                                 -relief raised -borderwidth 2 -indicatoron 1 -width 12
  pack  $frm.mbSU -side left
  entry $frm.entrySU -textvar MISSUEcho -state disabled -width 15
  pack  $frm.entrySU -side left -fill x -expand true
  set MISSUEcho "None"
  
  set m [menu $frm.mbSU.menu -tearoff 0]

  set SUList [open "|su.k LIST" r]   
  # -- eat the first two lines
  gets $SUList line
  # -- this is the format line -- need it's length to accurately know the range of characters
  #    to pull for the FILENAME.ST section
    gets $SUList line
    set length [string length [lindex $line 3]]

  while { [gets $SUList line] > -1 } \
    {
    # don't use SU: DONE
    if { [string first "su.k: DONE" $line] == -1 }\
      {
      # don't use line # bytes in # files
      if { [string first "bytes in" $line] == -1 }\
        {
	set ToSet [string range $line 28 [expr 28+$length-1]]
	set ToSet [lindex [split $ToSet .] 0]
        $m add command -label $ToSet -command "set MISSUEcho $ToSet" -font $fontset
	}
      }
    }  
  $m add command -label "None" -command "set MISSUEcho None"  -font $fontset
  catch { close  $SUList }
  }


#----------------------------------------------------------------#
# MISSetGB frm -- build the menu button and display for GrayBar command
#----------------------------------------------------------------#
proc MISSetGB { frm } \
  {
  global MISGBEcho

  set fontset {-adobe-courier-bold-r-normal--*-90*}
  menubutton $frm.mbGB -text "Grayscale Bar" -menu $frm.mbGB.menu \
                                 -relief raised -borderwidth 2 -indicatoron 1 -width 12
  pack  $frm.mbGB -side left
  entry $frm.entryGB -textvar MISGBEcho -state disabled -width 15
  pack  $frm.entryGB -side left -fill x -expand true
  set MISGBEcho "None"
  
  set m [menu $frm.mbGB.menu -tearoff 0]

  $m add command -label "None" -command "set MISGBEcho None" -font $fontset
  $m add command -label "Default" -command "set MISGBEcho Default" -font $fontset
  $m add command -label "Reverse" -command "set MISGBEcho Reverse" -font $fontset
  }



#==========================================================
# GetStationList -- This process loads the station menu
#		    button with the NEXRAD stations for
#		    the given state.	
#===========================================================
  proc GetStationList { } \
  {
	global data
  	global selected_state 
	global station_list
	global $data.frameMenu.radarFrame.idSelectionFrame.stationMenuButton.menu
	global $data.frameMenu.radarFrame.idSelectionFrame.entryWindowFrame.entryInitials
	global TempServerIndex
	global TempEntryIndex
	global NEXRAD_ID


	# Create input stream operator to STNLIST command
	set input_stream [open "|stnlist.k ST=$selected_state TYPE=N" r]

	# If no NEXRAD stations exist for a given state, set message and exit
	gets $input_stream line
	if {[string compare [lindex $line 0] "No"] == 0} \
	{
		# Set 1st item in drop down list to "No Stations" message

		$data.frameMenu.radarFrame.idSelectionFrame.stationMenuButton configure \
			-text "No Stations" \
			-indicatoron no \
			-state disabled
		return
	}

	# Clear the station_list variable
	$data.frameMenu.radarFrame.idSelectionFrame.stationMenuButton.menu delete 0 last

	for {set i 0} {$i < 20} {incr i} \
	{
		if {[info exists station_list($i)]} {unset station_list($i)}
	}

	set i 0
	while {[gets $input_stream line] > -1} \
	{
		global station_list

		# Break out of while-loop if end of listing is reached
		if {[string compare [lindex $line 0] "Number"] == 0} {break}

		# Set menu items to list obtains through STNLIST 
		set full_station_list($i) [string range $line 6 76]
		set station_list($i) [lindex $line 1]
		incr i
	} 

	# Load station menu with station ID's
	set i 0
	while {[info exists station_list($i)]} \
	{
		global $data.frameMenu.radarFrame.idSelectionFrame.stationMenuButton.menu

		$data.frameMenu.radarFrame.idSelectionFrame.stationMenuButton.menu add command \
			-label "$full_station_list($i)"\
			-command "$data.frameMenu.radarFrame.idSelectionFrame.entryWindowFrame.entryInitials delete 0 end; \
				  $data.frameMenu.radarFrame.idSelectionFrame.stationMenuButton configure \
					-text $station_list($i);\
				  set NEXRAD_ID $station_list($i); \
				  set station_exists true; \
				  set enMISSensorBand {}; \
				  set enMISDate {};
		  		  set MISEMag 1; \
		  		  MISModifyEMag; \
		  		  set MISLMag 1; \
		  		  MISModifyLMag; \
		  		  MISDoList $TempServerIndex $TempEntryIndex; \
		  		  update"
		incr i
	}

	# Close input operator
	while { [gets $input_stream line] != -1 } { }
	catch {close $input_stream}
  } 



#==========================================================
# VerifyStation -- This process checks to make sure that
#		   a given station exists.  If the station
#		   exists then 'station_exists' is set to
#		   true, otherwise it is set to false.
#===========================================================
  proc VerifyStation {inputID} \
  {
	global station_exists

	# Initialize return value
	set station_exists "true"

	# If 'inputID' is empty, display GUI error window
	if {[string compare $inputID ""] == 0} \
	{
		set station_exists "false"
		return
	}

	# Create input stream operator to STNLIST command and read first line
	set input_stream [open "|stnlist.k $inputID TYPE=N" r]
	gets $input_stream line

	# Display GUI error window if station does not exist
	if {[string compare [lindex $line 0] "No"] == 0} \
	{
		set station_exists "false"
	}

	# Close input operator
	while { [gets $input_stream line] != -1 } { }
	catch {close $input_stream}	
  }


#====================================================================
# RADAR_Options -- Add/Remove the RADAR options frame from
#		   the Image Selection GUI.
#
#	RADAR_Options INIT - creates all objects (call only once)
#	RADAR_Options ADD - packs outer frame
#	RADAR_Options REMOVE - unpacks outer frame
#
#====================================================================
proc RADAR_Options {action ServerIndex EntryIndex } \
{
	global data
	global RADAR_Options
  	global selected_state 
	global station_list
	global $data.frameMenu.radarFrame.idSelectionFrame.stationMenuButton.menu
	global $data.frameMenu.radarFrame.idSelectionFrame.entryWindowFrame.entryInitials
	global TempServerIndex
	global TempEntryIndex
	global NEXRAD_ID
	global MISLBox

   	# Tk widget commands moved to namespaces with version 8.1 and higher
   	# Expose the old private Tk commands.
   	if { $::tk_version >= 8.1 } \
   	{
	   ::tk::unsupported::ExposePrivateCommand tk*
   	}
	set TempServerIndex $ServerIndex
	set TempEntryIndex $EntryIndex

	#--Create all objects for RADAR options frame
	if {[string compare $action "INIT"] == 0} \
	{
		frame $data.frameMenu.radarFrame \
			-relief groove \
			-borderwidth 3 \
			-height 10

		#--Create label for main RADAR frame
		label $data.frameMenu.radarFrame.mainLabel \
			-text "Additional Controls for RADAR Data:" \
			-justify left
		pack $data.frameMenu.radarFrame.mainLabel \
			-side top \
			-anchor w \
			-padx 10 \
			-pady 5

		# Create frame for ID entry/selection widgets
		frame $data.frameMenu.radarFrame.idSelectionFrame \
			-borderwidth 2 \
			-relief groove
		pack $data.frameMenu.radarFrame.idSelectionFrame \
			-side top \
			-expand true \
			-fill x \
			-padx 5

		# Create frame for ID entry window widget
		frame $data.frameMenu.radarFrame.idSelectionFrame.entryWindowFrame
		pack $data.frameMenu.radarFrame.idSelectionFrame.entryWindowFrame \
			-side top

		# Create instruction label for ID entry window
  		label $data.frameMenu.radarFrame.idSelectionFrame.entryWindowFrame.entryLabel \
			-text "Enter NEXRAD station ID:" 
  		pack $data.frameMenu.radarFrame.idSelectionFrame.entryWindowFrame.entryLabel \
			-side left

		# Create entry window
  		entry $data.frameMenu.radarFrame.idSelectionFrame.entryWindowFrame.entryInitials \
			-textvariable Entry_Initials \
			-relief sunken \
			-borderwidth 3 \
			-highlightthickness 0 \
			-width 4 \
			-background white
  		pack $data.frameMenu.radarFrame.idSelectionFrame.entryWindowFrame.entryInitials \
			-side left \
			-pady 5 \
			-padx 5

		# Clear entry window
  		$data.frameMenu.radarFrame.idSelectionFrame.entryWindowFrame.entryInitials delete 0 end

		# Set entry window to upper case only
 		bind $data.frameMenu.radarFrame.idSelectionFrame.entryWindowFrame.entryInitials <KeyPress> \
			{ set Entry_Initials [string toupper $Entry_Initials] }
  		bindtags $data.frameMenu.radarFrame.idSelectionFrame.entryWindowFrame.entryInitials \
			[list Entry . $data.frameMenu.radarFrame.idSelectionFrame.entryWindowFrame.entryInitials all]

		# Bind the enter key to the entry window
  		bind $data.frameMenu.radarFrame.idSelectionFrame.entryWindowFrame.entryInitials <KeyPress-Return> \
		{
			$data.frameMenu.radarFrame.idSelectionFrame.stateMenuButton configure \
				-text State; \
			$data.frameMenu.radarFrame.idSelectionFrame.stationMenuButton configure \
				-text Station \
				-state disabled; \
			set NEXRAD_ID [$data.frameMenu.radarFrame.idSelectionFrame.entryWindowFrame.entryInitials get]; \
			set NEXRAD_ID [string toupper $NEXRAD_ID]; \
			VerifyStation $NEXRAD_ID; \
			set enMISSensorBand {}; \
			set enMISDate {}; \
		  	MISDoList $TempServerIndex $TempEntryIndex
		}

		# Place initial focus in entry window
  		tkTabToWindow $data.frameMenu.radarFrame.idSelectionFrame.entryWindowFrame.entryInitials

		# Create 'hit enter' label
		label $data.frameMenu.radarFrame.idSelectionFrame.entryWindowFrame.hitEnterLabel \
			-text "then hit 'Enter'"
		pack $data.frameMenu.radarFrame.idSelectionFrame.entryWindowFrame.hitEnterLabel \
			-side left

		# Create frame for 'or' label
		frame $data.frameMenu.radarFrame.idSelectionFrame.orLabelFrame
		pack $data.frameMenu.radarFrame.idSelectionFrame.orLabelFrame \
			-side top \
			-pady 10

		# Create 'or' label
		label $data.frameMenu.radarFrame.idSelectionFrame.orLabelFrame.orLabel \
			-text " --- or --- " 
		pack $data.frameMenu.radarFrame.idSelectionFrame.orLabelFrame.orLabel \
			-side left

		# Create 'Choose state' label
		label $data.frameMenu.radarFrame.idSelectionFrame.chooseStateLabel \
			-text " Choose a state... "
 		pack $data.frameMenu.radarFrame.idSelectionFrame.chooseStateLabel \
			-side left \
			-pady 10

		# Load variables with all of the state abreviations
		  set states "AL AK AR AZ CA CO CT DE DC FL GA HI ID IL IN \
		  	      IA KS KY LA ME MD MA MI MN MS MO MT NE NH NM \
		  	      NJ NY NC ND OH OK OR PA RI SC SD TN TX UT VA \
		  	      VT WA WI WV WY GU PR VI"
		
		# Create state menu button
		  menubutton $data.frameMenu.radarFrame.idSelectionFrame.stateMenuButton \
			-text "State" \
			-indicatoron yes \
			-width 6 \
			-borderwidth 2 \
			-relief raised \
			-menu $data.frameMenu.radarFrame.idSelectionFrame.stateMenuButton.stateMenu
		  pack $data.frameMenu.radarFrame.idSelectionFrame.stateMenuButton \
			-side left

		# Create state menu
		  menu $data.frameMenu.radarFrame.idSelectionFrame.stateMenuButton.stateMenu \
			-tearoff no

		# Load states menu
		  set i 1
		  set new_column no
		  foreach state $states \
		  {
		  	$data.frameMenu.radarFrame.idSelectionFrame.stateMenuButton.stateMenu add command \
				-label $state \
				-columnbreak $new_column \
				-command "$data.frameMenu.radarFrame.idSelectionFrame.stateMenuButton configure \
						-text $state; \
					  set selected_state $state; \
					  $data.frameMenu.radarFrame.idSelectionFrame.stationMenuButton configure \
						-text {Station} \
						-indicatoron yes \
						-state normal; \
					  $data.frameMenu.radarFrame.idSelectionFrame.entryWindowFrame.entryInitials \
						 delete 0 end; \
					  GetStationList"
	
			# Start a new column after each 5 states
			if {$i == 5} \
			{
				set new_column yes
				set i 1
			} \
			else \
			{
				set new_column no
				incr i	
			}
		  }


		# Create 'Choose a station' label
  		label $data.frameMenu.radarFrame.idSelectionFrame.chooseStationLabel \
			-text " And then choose a station..."
  		pack $data.frameMenu.radarFrame.idSelectionFrame.chooseStationLabel \
			-side left

		# Create station menu button
		  menubutton $data.frameMenu.radarFrame.idSelectionFrame.stationMenuButton \
			-text "Station" \
			-indicatoron yes \
			-width 10 \
			-borderwidth 2 \
			-relief raised \
			-state disabled \
			-menu $data.frameMenu.radarFrame.idSelectionFrame.stationMenuButton.menu
		  pack $data.frameMenu.radarFrame.idSelectionFrame.stationMenuButton \
			-side left \
			-padx 5

		# Create menu to be filled during call to GetStationList
		  menu $data.frameMenu.radarFrame.idSelectionFrame.stationMenuButton.menu \
			-tearoff no

   		#--Create checkbutton for standard RADAR options
      		checkbutton $data.frameMenu.radarFrame.radarOptionsCheckButton \
			-text {Use Standard RADAR Settings (color enhancements, bar scale, maps, etc.)} \
			-command {if {[string compare $RADAR_Options "ON"] == 0} \
				  {
					$data.frameMenu.fmMap.cbUseMap deselect
					$data.frameMenu.fmMap.cbUseMap configure \
						-state disabled
					$data.frameMenu.fmMap.btConfig configure \
						-state disabled
				  } \
				  else \
				  {
					$data.frameMenu.fmMap.cbUseMap configure \
						-state normal
					$data.frameMenu.fmMap.btConfig configure \
						-state normal
				  }
				 } \
			-variable RADAR_Options \
			-onvalue ON \
			-offvalue OFF \
			-highlightthickness 0
      		pack $data.frameMenu.radarFrame.radarOptionsCheckButton \
			-side top \
			-padx 5 \
			-pady 10

	} \
	elseif {[string compare $action "ADD"] == 0}\
	{
		#--Clear the list box
		$MISLBox delete 0 end

		#--Deselect and disable the map controls
		$data.frameMenu.fmMap.cbUseMap deselect
		$data.frameMenu.fmMap.cbUseMap configure \
			-state disabled
		$data.frameMenu.fmMap.btConfig configure \
			-state disabled

		#--Reinitialize the ID/State/Station entry widgets
		$data.frameMenu.radarFrame.idSelectionFrame.entryWindowFrame.entryInitials \
			delete 0 end
		$data.frameMenu.radarFrame.idSelectionFrame.stateMenuButton configure \
			-text "State"
		$data.frameMenu.radarFrame.idSelectionFrame.stationMenuButton configure \
			-text "Station" \
			-state disabled

		#--Set the size of the Image Selection GUI to accept the RADAR frame
		wm geometry .tlImageSelect 650x760
		.tlImageSelect.nbImageDisp configure \
			 -height 700
		pack $data.frameMenu.radarFrame \
			-side top \
			-padx 10 \
			-fill x \
			-expand true

		#--Set standard RADAR options check button to 'ON'
      		$data.frameMenu.radarFrame.radarOptionsCheckButton select
	} \
	elseif {[string compare $action "REMOVE"] == 0} \
	{
		$data.frameMenu.fmMap.cbUseMap configure \
			-state normal
		$data.frameMenu.fmMap.btConfig configure \
			-state normal
		pack forget $data.frameMenu.radarFrame
		set RADAR_Options "NO"
		wm geometry .tlImageSelect 650x625
		.tlImageSelect.nbImageDisp configure \
			 -height 565
	}
}

################################################################
# CREATE_BAND_LIST - Create a list of available bands for the 
#		     for the given image.
################################################################
proc CREATE_BAND_LIST {groupDatasetPosition} \
{
   global band_list
   global num_bands

   # Call IMGLIST to get a list of bands
   set inputOperator [open "|imglist.k $groupDatasetPosition FORM=EXP" r]
   set counter 0
   array set band_list {}
   while {[gets $inputOperator line] > -1} \
   {
	if {[string compare [lindex $line 0] "Band:"] == 0} \
	{
	   incr counter
	   set band_list($counter) [string range $line 0 52]
	}
   }
   catch {close $inputOperator}

   set num_bands $counter
} 


###################################################################
# FILL_LIST_BOX - 
###################################################################
proc FILL_LIST_BOX {} \
{
   global band_list 
   global MISLBox
   global num_bands
   global MISSelectionChange

   $MISLBox delete 0 end

   for {set i 1} {$i <= $num_bands} {incr i} \
   {
	$MISLBox insert end $band_list($i)
   }

#   $MISLBox selection set 0
#   MISClick
#   set MISSelectionChange 1
}
   



#########################################################
# Directory Selector TCL version 1.5
#
# Daniel Roche, <daniel.roche@bigfoot.com>
#
# thanks to :
#  Cyrille Artho <cartho@netlink.ch> for the 'saving pwd fix'
#  Terry Griffin <terryg@axian.com> for <Return> key bindings on buttons.
#  Kenneth Kehl  <Kenneth.Kehl@marconiastronics.com> for blocking at end of dir tree
#  Michael Barth <m.barth@de.bosch.com> for finding the "-fill on image" problem 
#  Mike Avery <avery@loran.com> for finding the "myfont already exist" problem
#  Branko Rogic <b.rogic@lectra.com> for gif icons, parent and background options
#  Reinhard Holler <rones@augusta.de> for colors ,font and autovalid options
#
# ---------------------------------------------
# 
# tk_getDirectory [option value ...]
#
#  options are :
#   [-initialdir dir]     display in dir
#   [-title string]       make string title of dialog window
#   [-ok string]          make string the label of OK button
#   [-open string]        make string the label of OPEN button
#   [-cancel string]      make string the label of CANCEL button
#   [-msg1 string]        make string the label of the first directory message
#   [-msg2 string]        make string the label of the second directory message
#   [-parent string]      make string the parent of tkgetdir for transient window
#   [-geometry string]    make string the geometry (WxH+X+Y)
#   [-listbg string]      make string the list and entry background color
#   [-listfg string]      make string the list and entry foreground color
#   [-selectbg string]    make string the background color for selected directory
#   [-selectfg string]    make string the foreground color for selected directory
#   [-hilightbg string]   make string the background color for highlighted dir
#   [-hilightfg string]   make string the foreground color for highlighted dir
#   [-endcolor string]    make string the color for the last directory flag
#   [-autovalid]          if it is set : selecting no directory in the window
#                          will return the directory the window shows currently
#   [-font string]        make string the font name or font description
#                          for directory window
#   [-listfont string]    make string the font name or font description
#			   for list box contents
#
#########################################################

namespace eval tkgetdir {
    variable drives
    variable fini
    variable svwd
    variable msg
    variable geometry
    variable parent
    variable colors
    variable font
    variable myfont
    variable listfont
    variable autovalid
    variable file_name 
    variable position 
    variable image_type 
    variable directory
    
    namespace export tk_getDirectory
}

###########################################################
proc tkgetdir::tk_getDirectory {args} {
    global tcl_platform
    global current_directory
    global initial_directory
    global directory_name
    global newdir
    global sorted_directory_list
    global sorted_area_file_list
    global sorted_stand_modis_file_list
    global current_selection
    global remote_flag


    #
    # default
    #
    set tkgetdir::msg(title) "Directory Selector"
    set tkgetdir::msg(ldir) "Directory:"
    set tkgetdir::msg(ldnam) "Directory Name:"
    set tkgetdir::msg(open) " Ok "
    set tkgetdir::msg(expand) "Open"
    set tkgetdir::msg(cancel) "Cancel"
    set tkgetdir::geometry "500x250"
    set tkgetdir::colors(lstfg) "#000000"
    if [string compare $tcl_platform(platform) windows] {
	set tkgetdir::colors(lstbg) ""
	set tkgetdir::font {-family lucida -size 14 -weight bold}
	set tkgetdir::listfont {-family lucida -size 14 -weight bold}
    } else {
	set tkgetdir::colors(lstbg) "#FFFFFF"
	set tkgetdir::font {-family courier -size 10}
	set tkgetdir::listfont {-family courier -size 10}
    }
    set tkgetdir::colors(hilfg) "#FFFF00"
    set tkgetdir::colors(hilbg) "#808080"
    set tkgetdir::colors(selfg) "#FFFFFF"
    set tkgetdir::colors(selbg) "#000080"
    set tkgetdir::colors(endcol) "#FF8000"
    set tkgetdir::parent ""
    set tkgetdir::autovalid 0
    set tkgetdir::svwd [pwd]
    set remote_flag LOCAL

    #
    # arguments
    #
    set ind 0
    set max [llength $args]
    while { $ind < $max } {
	switch -exact -- [lindex $args $ind] {
	    "-initialdir" {
		incr ind
		set current_directory [lindex $args $ind]
		set initial_directory $current_directory
		set newdir $current_directory
		
		# Strip drive letter off of path if Interix
		set InputOperator [open "|uname" r]
		gets $InputOperator uname_results
		catch {close $InputOperator}
		if {[string match $uname_results "Windows_NT"]} \
		{
		   set current_directory [string range $current_directory 3 end]
		   set initial_directory $current_directory
		   set newdir $current_directory
		}
		cd "$current_directory"
		incr ind
	    }
	    "-title" {
		incr ind
		set tkgetdir::msg(title) [lindex $args $ind]
		incr ind
	    }
	    "-ok" {
		incr ind
		set tkgetdir::msg(open) [lindex $args $ind]
		incr ind
	    }
	    "-open" {
		incr ind
		set tkgetdir::msg(expand) [lindex $args $ind]
		incr ind
	    }
	    "-cancel" {
		incr ind
		set tkgetdir::msg(cancel) [lindex $args $ind]
		incr ind
	    }
	    "-msg1" {
		incr ind
		set tkgetdir::msg(ldir) [lindex $args $ind]
		incr ind
	    }
	    "-msg2" {
		incr ind
		set tkgetdir::msg(ldnam) [lindex $args $ind]
		incr ind
	    }
	    "-geometry" {
		incr ind
		set tkgetdir::geometry [lindex $args $ind]
		incr ind
	    }
	    "-parent" {
		incr ind
		set tkgetdir::parent [lindex $args $ind]
		incr ind
	    }
	    "-listfg" {
		incr ind
		set tkgetdir::colors(lstfg) [lindex $args $ind]
		incr ind
	    }
	    "-listbg" {
		incr ind
		set tkgetdir::colors(lstbg) [lindex $args $ind]
		incr ind
	    }
	    "-selectfg" {
		incr ind
		set tkgetdir::colors(selfg) [lindex $args $ind]
		incr ind
	    }
	    "-selectbg" {
		incr ind
		set tkgetdir::colors(selbg) [lindex $args $ind]
		incr ind
	    }
	    "-hilightfg" {
		incr ind
		set tkgetdir::colors(hilfg) [lindex $args $ind]
		incr ind
	    }
	    "-hilightbg" {
		incr ind
		set tkgetdir::colors(hilbg) [lindex $args $ind]
		incr ind
	    }
	    "-endcolor" {
		incr ind
		set tkgetdir::colors(endcol) [lindex $args $ind]
		incr ind
	    }
	    "-autovalid" {
		incr ind
		set tkgetdir::autovalid 1
	    }
	    "-font" {
		incr ind
		set tkgetdir::font [lindex $args $ind]
		incr ind
	    }
	    "-listfont" {
		incr ind
		set tkgetdir::listfont [lindex $args $ind]
		incr ind
	    }
	    default {
		puts "unknown option [lindex $args $ind]"
		return ""
	    }
	}
    }
    
    #
    # variables and data
    #
    set tkgetdir::fini 0
    
    image create photo b_up -data {
	R0lGODlhFgATAMIAAHt7e9/fX////gAAAK6uSv///////////yH+Dk1hZGUgd2l0aCBHSU1QACH5
	BAEAAAcALAAAAAAWABMAAANVeArcoDBKEKoNT2p6b9ZLJzrkAQhoqq4qMJxi3LnwRcjeK9jDjWM6
	C2FA9Mlou8CQWMQhO4Nf5XmJSqkW6w9bYXqZFq40HBzPymYyac1uDA7fuJyZAAA7
    }

    image create photo b_dir -data {
	R0lGODlhEAAQAMIAAHB/cN/fX////gAAAP///////////////yH+Dk1hZGUgd2l0aCBHSU1QACH5
	BAEAAAQALAAAAAAQABAAAAM2SLrc/jA2QKkEIWcAsdZVpQBCaZ4lMBDk525r+34qK8x0fOOwzfcy
	Xi2IG4aOoRVhwGw6nYQEADs=
    }

    image create photo b_dir_remote -data {
	R0lGODlhEAAQAMIAAHB/cN/fX////gAAAP///////////////yH+Dk1hZGUgd2l0aCBHSU1QACH5
	BAEAAAQALAAAAAAQABAAAAM2SLrc/jA2QKkEIWcAsdZVpQBCaZ4lMBDk525r+34qK8x0fOOwzfcy
	Xi2IG4aOoRVhwGw6nYQEADs=
    }

    if {[lsearch -exact $tkgetdir::font -family] >= 0} \
    {
	eval font create dlistfont $tkgetdir::font
	set tkgetdir::myfont dlistfont
    } \
    else \
    {
	set tkgetdir::myfont $tkgetdir::font
    }

    #
    # widgets
    #
    toplevel .dirsel
#    grab set .dirsel
    wm geometry .dirsel $tkgetdir::geometry
#    if {$tkgetdir::parent != ""} {
#        set par $tkgetdir::parent
#	set xOrgWin [expr [winfo rootx $par] + [winfo width $par] / 2 ]
#	set yOrgWin [expr [winfo rooty $par] + [winfo height $par] / 2 ]
#	wm geometry .dirsel +$xOrgWin+$yOrgWin
#	wm transient .dirsel $tkgetdir::parent
#    }
    wm title .dirsel $tkgetdir::msg(title)

#ml    event add <<RetEnt>> <Return> <KP_Enter>

    frame .dirsel.f1 -relief flat -borderwidth 0
    frame .dirsel.f1.f1a -relief flat -borderwidth 0
    frame .dirsel.f2 -relief sunken -borderwidth 2
    frame .dirsel.f4 -relief flat -borderwidth 0
    
    pack .dirsel.f1 -fill x    
    pack .dirsel.f2 -fill both -expand 1 -padx 6
    pack .dirsel.f4 
    
    label .dirsel.f1.lab -text $tkgetdir::msg(ldir)
    button .dirsel.f1.up -image b_up -command tkgetdir::UpDir
    bind .dirsel.f1.up <<RetEnt>> {.dirsel.f1.up invoke}
    
    label .dirsel.f1.f1a.directoryWindow \
    	-justify left \
	-anchor w \
	-foreground black \
	-relief sunken \
	-borderwidth 2 \
    	-text $current_directory
    
    pack .dirsel.f1.up -side right -padx 4 -pady 4
    pack .dirsel.f1.lab -side left -padx 4 -pady 4
    pack .dirsel.f1.f1a -side left -expand true -fill x -pady 5
    pack .dirsel.f1.f1a.directoryWindow \
    	-side top \
	-fill x \
	-expand true \
	-padx 5
    
    canvas .dirsel.f2.cv -borderwidth 0 -yscrollcommand ".dirsel.f2.sb set"
    if { $tkgetdir::colors(lstbg) != "" } {
	.dirsel.f2.cv configure -background $tkgetdir::colors(lstbg)
    }
    scrollbar .dirsel.f2.sb -command ".dirsel.f2.cv yview"
    set scw 16
    place .dirsel.f2.cv -x 0 -relwidth 1.0 -width [expr -$scw ] -y 0 \
	    -relheight 1.0
    place .dirsel.f2.sb -relx 1.0 -x [expr -$scw ] -width $scw -y 0 \
	    -relheight 1.0
    unset scw

    # Bindings for local directories   
    .dirsel.f2.cv bind TXT <Double-Button-1> tkgetdir::DoubleClickItem
    .dirsel.f2.cv bind IMG <Double-Button-1> tkgetdir::DoubleClickItem
#    .dirsel.f2.cv bind TXT <Enter> tkgetdir::EnterItem
#    .dirsel.f2.cv bind TXT <Leave> tkgetdir::LeaveItem
#    .dirsel.f2.cv bind IMG <Enter> tkgetdir::EnterItem
#    .dirsel.f2.cv bind IMG <Leave> tkgetdir::LeaveItem

    # Bindings for the remote directory
#    .dirsel.f2.cv bind REMOTE_DIR_IMG <Enter> tkgetdir::EnterItem
#    .dirsel.f2.cv bind REMOTE_DIR_IMG <Leave> tkgetdir::LeaveItem
    .dirsel.f2.cv bind REMOTE_DIR_IMG <Double-Button-1> \
	tkgetdir::DoubleClickRemoteDir
#    .dirsel.f2.cv bind REMOTE_DIR_TXT <Enter> tkgetdir::EnterItem
#    .dirsel.f2.cv bind REMOTE_DIR_TXT <Leave> tkgetdir::LeaveItem
    .dirsel.f2.cv bind REMOTE_DIR_TXT <Double-Button-1> \
	tkgetdir::DoubleClickRemoteDir

    # Bindings for remote groups
#    .dirsel.f2.cv bind REMOTE_GROUP_IMG <Enter> tkgetdir::EnterItem
#    .dirsel.f2.cv bind REMOTE_GROUP_IMG <Leave> tkgetdir::LeaveItem
    .dirsel.f2.cv bind REMOTE_GROUP_IMG <Double-Button-1> \
	"tkgetdir::DoubleClickRemoteGroup X"
#    .dirsel.f2.cv bind REMOTE_GROUP_TXT <Enter> tkgetdir::EnterItem
#    .dirsel.f2.cv bind REMOTE_GROUP_TXT <Leave> tkgetdir::LeaveItem
    .dirsel.f2.cv bind REMOTE_GROUP_TXT <Double-Button-1> \
	"tkgetdir::DoubleClickRemoteGroup X"

    # Bindings for remote datasets
#    .dirsel.f2.cv bind REMOTE_DATASET_IMG <Enter> tkgetdir::EnterItem
#    .dirsel.f2.cv bind REMOTE_DATASET_IMG <Leave> tkgetdir::LeaveItem
    .dirsel.f2.cv bind REMOTE_DATASET_IMG <Double-Button-1> \
	tkgetdir::DoubleClickRemoteDataset
#    .dirsel.f2.cv bind REMOTE_DATASET_TXT <Enter> tkgetdir::EnterItem
#    .dirsel.f2.cv bind REMOTE_DATASET_TXT <Leave> tkgetdir::LeaveItem
    .dirsel.f2.cv bind REMOTE_DATASET_TXT <Double-Button-1> \
	tkgetdir::DoubleClickRemoteDataset

    # Bindings for remote image files
#    .dirsel.f2.cv bind REMOTE_IMAGEFILE_TXT <Enter> tkgetdir::EnterItem
#    .dirsel.f2.cv bind REMOTE_IMAGEFILE_TXT <Leave> tkgetdir::LeaveItem
    .dirsel.f2.cv bind REMOTE_IMAGEFILE_TXT <Button-1> {tkgetdir::ClickItem PRESS}
    .dirsel.f2.cv bind REMOTE_IMAGEFILE_TXT <ButtonRelease-1> {tkgetdir::ClickItem RELEASE}
    .dirsel.f2.cv bind REMOTE_IMAGEFILE_TXT <Double-Button-1> \
	tkgetdir::DoubleClickRemoteImageFile

    # Bindings for local image files
#    .dirsel.f2.cv bind FILES <Enter> tkgetdir::EnterItem
#    .dirsel.f2.cv bind FILES <Leave> tkgetdir::LeaveItem
    .dirsel.f2.cv bind FILES <Button-1> {tkgetdir::ClickItem PRESS}
    .dirsel.f2.cv bind FILES <ButtonRelease-1> {tkgetdir::ClickItem RELEASE}
    .dirsel.f2.cv bind FILES <Double-Button-1> \
		{CreateDataset \
			$sorted_directory_list \
			$sorted_area_file_list \
			$sorted_stand_modis_file_list \
			[.dirsel.f2.cv itemcget \
			   [.dirsel.f2.cv find withtag current] -text] \
			$newdir}

    button .dirsel.f4.open \
	-text $tkgetdir::msg(open) \
	-command {if {[string compare $remote_flag "REMOTE"] == 0} \
		  {
			tkgetdir::DoubleClickRemoteImageFile
		  } \
		  else \
		  {
			if {[info exists current_selection]} \
			{
		   	   CreateDataset \
			   	$sorted_directory_list \
			   	$sorted_area_file_list \
			   	$sorted_stand_modis_file_list \
			   	$current_selection \
			   	$newdir
			}
		  }
		 }

#    bind .dirsel.f4.open <<RetEnt>> {.dirsel.f4.open invoke}

    button .dirsel.f4.expand -text $tkgetdir::msg(expand) -command tkgetdir::DownDir
    bind .dirsel.f4.expand <<RetEnt>> {.dirsel.f4.expand invoke}

    button .dirsel.f4.cancel \
	-text $tkgetdir::msg(cancel) \
	-command {destroy .dirsel}

    bind .dirsel.f4.cancel <<RetEnt>> {.dirsel.f4.cancel invoke}

    button .dirsel.f4.help \
	-text "Help" \
   	-command "execGUI helplite HELPDISPLAY"

    pack .dirsel.f4.open -side left -padx 10 -pady 4

    pack .dirsel.f4.cancel -side left -padx 5 -pady 5
    pack .dirsel.f4.help \
	-padx 5 \
	-pady 5
    
    bind .dirsel.f1 <Destroy> tkgetdir::CloseDirSel

    #
    # realwork
    #
    tkgetdir::ShowDir $initial_directory
    
    #
    # wait user
    #
#    tkwait variable tkgetdir::fini

    if { $tkgetdir::fini == 1 } \
    {
	set retval $newdir
    } \
    else \
    {
	set retval ""
    }
    
    if {$tkgetdir::fini == -1} {destroy .dirsel}
    
    return $retval
}

###########################################################
proc tkgetdir::CloseDirSel {} {
    set wt [font names]
    if {[lsearch -exact $wt dlistfont] >= 0} {
	font delete dlistfont 
    }
    event delete <<RetEnt>>
    cd $tkgetdir::svwd
    set tkgetdir::fini -1
}

###########################################################
proc tkgetdir::ShowDir {curdir} {

    global tcl_platform
    global current_directory
    global initial_directory
    global sorted_directory_list
    global sorted_area_file_list
    global sorted_stand_modis_file_list

    variable drives 

    set current_directory $curdir
    cd $curdir
   
    set hi1 [font metrics $tkgetdir::myfont -linespace]
    set hi2 [image height b_dir]
    if { $hi1 > $hi2 } \
    {
	set hi $hi1
    } \
    else \
    {
	set hi $hi2
    }
    set wi1 [image width b_dir]
    incr wi1 4
    set wi2 [winfo width .dirsel.f2.cv]
    
    # Initialize lists for directories and files
    set directory_list [list]
    set area_file_list [list]
    set stand_modis_file_list [list]

    # Time for the watch
    showBusyStatus

    # If this is the initial directory add "RemoteData" as
    # a virtual directory
    if {[string match "*/mclite/data" $current_directory]} \
    {
	lappend directory_list "RemoteData"
    }

    # Get local files and directories
    foreach file [ glob -nocomplain * ] \
    {
	if [ file isdirectory [string trim $file "~"] ] \
	{ 
	    # This is a directory so add it to the directory list
	    lappend directory_list $file
	} \
	elseif [ file isfile [string trim $file "~"] ] \
	{
	   # Use LWU to read part of the file to determine what
	   # type of file that it is.
	   set inputOperator [open "|lwu.k LIST $curdir/$file" r]
	   gets $inputOperator line
    	   catch {close $inputOperator}

	   catch \
	   {
		# Trim the leading zeros (platform independance issue)
	   	if {[string compare [string trim [lindex $line 5] "0"] "4"] == 0} \
	   	{
		   # This is an area file so add it to the area list
		   lappend area_file_list $file
	   	} \
	   	elseif {[string compare [string range $file 0 4] "MOD02"] == 0} \
	   	{
		   # This is a standard MODIS file so add it to the modis list
	 	   lappend stand_modis_file_list $file
	   	}
	   }
	}
    }

    # Sort the lists
    set sorted_directory_list [lsort $directory_list]
    set sorted_area_file_list [lsort $area_file_list]
    set sorted_stand_modis_file_list [lsort $stand_modis_file_list]

    # Initialize counter variables for the various file types
    set num_directories 0
    set num_area_files 0
    set num_stand_modis_files 0

    .dirsel.f2.cv delete all
    set ind 0

    # Adjust the position of the text wi1 with an offset.
    if { $hi1 < $hi2 } \
    {
	set offset [expr $hi2 - $hi1]
    } \
    else \
    {
	set offset 0
    }
 
    # Place directories into list box
    foreach file $sorted_directory_list \
    {
	if {[string compare $file "RemoteData"] == 0} \
	{
	   set image_file b_dir_remote
	   set image_tag REMOTE_DIR_IMG
	   set text_tag REMOTE_DIR_TXT
	} \
	else \
	{
	   set image_file b_dir
	   set image_tag IMG
	   set text_tag TXT
	}

#ml	if [ file isdirectory $file ] \
#ml	{ 
	    .dirsel.f2.cv create image 2 [expr $ind * $hi] \
		    -anchor nw -image $image_file -tags $image_tag
	    .dirsel.f2.cv create text $wi1 [expr ($ind * $hi) + $offset] \
		    -anchor nw -text $file -fill $tkgetdir::colors(lstfg) \
		    -tags $text_tag \
		    -font $tkgetdir::listfont
	    set ind [ expr $ind + 1 ]

	    incr num_directories
#ml	}
    }

    # Place area files into list box
    foreach file $sorted_area_file_list \
    {
	if {[ file isfile $file ]} \
	{
	    .dirsel.f2.cv create text 5 [expr ($ind * $hi) + $offset] \
		    -anchor nw -text $file -fill $tkgetdir::colors(lstfg) \
		    -tags FILES \
		    -font $tkgetdir::listfont
	    set ind [ expr $ind + 1 ]

	    incr num_area_files
	}
    }

    # Place standard MODIS files into list box
    foreach file $sorted_stand_modis_file_list \
    {
	if {[ file isfile $file ]} \
	{
	    .dirsel.f2.cv create text 5 [expr ($ind * $hi) + $offset] \
		    -anchor nw -text $file -fill $tkgetdir::colors(lstfg) \
		    -tags FILES \
		    -font $tkgetdir::listfont
	    set ind [ expr $ind + 1 ]

	    incr num_stand_modis_files
	}
    }

    set ha [expr $ind * $hi]
    .dirsel.f2.cv configure -scrollregion [list 0 0 $wi2 $ha]
    .dirsel.f2.cv yview moveto 0.0
    
    set curlst [file split $curdir]
    set nbr [llength $curlst]
    
    incr nbr -2
    for {set ind $nbr} {$ind >= 0} {incr ind -1} {
	set tmplst [ lrange $curlst 0 $ind] 
	set tmpdir [ eval file join $tmplst] 
    }
    if {[info exist drives] == 0} {set drives [file volume]}

    # Restore cursor
    showIdleStatus  

}

###########################################################
proc tkgetdir::UpDir {} {

    global current_directory
    global newdir
    global remote_flag
    global current_selection

    catch {unset current_selection}

    # Set 'remote_flag' to LOCAL since we can no longer be 
    # viewing a list of remote image files since we have
    # hit the 'UpDir' button.
    set remote_flag LOCAL

    # If we are currently in a virtual directory make the appropriate changes
    if {[string match "*/RemoteData/*/*" $current_directory]} \
    {
	# We are currently viewing a list of images so go back to remote datasets
	set last_slash_index [string last "/" $current_directory]
	set temp1 [string range $current_directory 0 \
		    [expr {$last_slash_index - 1}]]
	set newdir $temp1
	set last_slash_index [string last "/" $temp1]
        set temp1 [string range $temp1 [expr {$last_slash_index + 1}] end]
    	tkgetdir::DoubleClickRemoteGroup $temp1
    } \
    elseif {[string match "*/RemoteData/*" $current_directory]} \
    {
	# We are currently viewing a list of datasets so go back to remote groups
	tkgetdir::DoubleClickRemoteDir
    } \
    else \
    {
	# We are not in any virtual directories so proceed as normal
    	set curdir $current_directory; #ml
    	set curlst [file split $curdir]
	set nbr [llength $curlst]
    	if { $nbr < 2 } {return}
    	set tmp [expr $nbr - 2]
    	set newlst [ lrange $curlst 0 $tmp ]
    	set newdir [ eval file join $newlst ]

    	tkgetdir::ShowDir $newdir
 
    	.dirsel.f1.f1a.directoryWindow configure \
	   -text $newdir
   }
}


###########################################################
proc tkgetdir::DownDir {} {

    global current_directory
    global directory_name
    global newdir

    set curdir $current_directory; #ml
    set nnam $directory_name; #ml
    set newdir [ file join $curdir $nnam ]

    # change 07/19/99
    # If there are more dirs, permit display of one level down.
    # Otherwise, block display and hilight selection in red.
    set areDirs 0
    foreach f [glob -nocomplain [file join $newdir *]] {
	if {[file isdirectory $f]} {
	    set areDirs 1
	    break
	}
    } 

    # Update directory display window
    .dirsel.f1.f1a.directoryWindow configure \
	-text $newdir

    tkgetdir::ShowDir $newdir
}

###########################################################
proc tkgetdir::EnterItem {} {
    global tcl_platform

    set id [.dirsel.f2.cv find withtag current]
    set wt [.dirsel.f2.cv itemcget $id -tags]

    if {[lsearch -exact $wt IMG] >= 0 || \
	[lsearch -glob $wt "REMOTE*IMG"] >= 0 } \
    {
	set id [.dirsel.f2.cv find above $id]
    }

    .dirsel.f2.cv itemconfigure $id \
	-fill $tkgetdir::colors(hilfg)
    set bxr [.dirsel.f2.cv bbox $id]
    eval .dirsel.f2.cv create rectangle $bxr \
	-fill $tkgetdir::colors(hilbg) \
	-outline $tkgetdir::colors(hilbg) \
	-tags HILIT
    .dirsel.f2.cv lower HILIT
}

###########################################################
proc tkgetdir::LeaveItem {} {
    .dirsel.f2.cv delete HILIT
    set id [.dirsel.f2.cv find withtag current]
    set wt [.dirsel.f2.cv itemcget $id -tags]

    if {[lsearch -exact $wt IMG] >= 0 || \
	[lsearch -glob $wt REMOTE*IMG] >= 0} \
    {
	set id [.dirsel.f2.cv find above $id]
    }

    set wt [.dirsel.f2.cv itemcget $id -tags]
    
    if {[lsearch -exact $wt HASBOX] >= 0} \
    {
	.dirsel.f2.cv itemconfigure $id -fill $tkgetdir::colors(selfg)
    } \
    else \
    {
	.dirsel.f2.cv itemconfigure $id -fill $tkgetdir::colors(lstfg)
    }
}

###########################################################
proc tkgetdir::ClickItem {action} {

    global directory_name
    global current_selection


    # Set press and release positions
    if {[string compare $action "PRESS"] == 0} \
    {
	.dirsel.f2.cv dtag beg
	.dirsel.f2.cv addtag beg withtag current
    } \
    else \
    {
	.dirsel.f2.cv dtag end
	.dirsel.f2.cv addtag end withtag current
    }

    # Tag currently selected item
    .dirsel.f2.cv dtag current_item
    .dirsel.f2.cv addtag current_item withtag current

#    .dirsel.f2.cv delete HILIT

    # put old selected item in normal state
    .dirsel.f2.cv itemconfigure BOX \
	  -fill $tkgetdir::colors(lstbg) \
	  -outline $tkgetdir::colors(lstbg) 
#    .dirsel.f2.cv delete BOX
    set id [.dirsel.f2.cv find withtag HASBOX]
    .dirsel.f2.cv itemconfigure $id -fill $tkgetdir::colors(lstfg)
    .dirsel.f2.cv dtag HASBOX HASBOX

    # put new selected item in selected state
    set id [.dirsel.f2.cv find withtag current]
    set wt [.dirsel.f2.cv itemcget $id -tags]
    if {[lsearch -exact $wt IMG] >= 0} {set id [.dirsel.f2.cv find above $id]}
     set bxr [.dirsel.f2.cv bbox $id]
    .dirsel.f2.cv addtag HASBOX withtag $id
    .dirsel.f2.cv itemconfigure $id -fill $tkgetdir::colors(selfg)
    eval .dirsel.f2.cv create rectangle $bxr \
	    -fill $tkgetdir::colors(selbg) -outline $tkgetdir::colors(selbg) \
	    -tags BOX
    .dirsel.f2.cv lower BOX
    set nam [.dirsel.f2.cv itemcget $id -text]
    set directory_name $nam

    set current_selection $nam

}

###########################################################
proc tkgetdir::DoubleClickItem {} {

    global directory_name
    global current_selection

    catch {unset current_selection}

    set id [.dirsel.f2.cv find withtag current]

    # If the image was clicked on, increment to next item
    set type [.dirsel.f2.cv type $id]
    if {[string compare $type "image"] == 0} {incr id}

    set nam [.dirsel.f2.cv itemcget $id -text]
    set directory_name $nam
    tkgetdir::DownDir
}

###########################################################
proc tkgetdir::DoubleClickRemoteDir {} \
{
   global initial_directory
   global env
   global current_selection

   catch {unset current_selection}

   # There is a 'dummy' directory in .../mclite/data called 'Remote Data'.
   # This routine is called if this directory is double-clicked.
   # A file called 'mclite_remote' is then read and a list of machine
   # names containing public data is displayed.

   global current_directory
   
   set id [.dirsel.f2.cv find withtag current]

   # If the folder image was clicked on, increment to next item
   set type [.dirsel.f2.cv type $id]
   if {[string compare $type "image"] == 0} {incr id}

   set current_directory "$initial_directory/RemoteData"

   # Add the new directory to the path in the directory window
   .dirsel.f1.f1a.directoryWindow configure \
	-text $current_directory
	
   set ind 0
   set hi1 [font metrics $tkgetdir::myfont -linespace]
   set hi2 [image height b_dir_remote]
   if { $hi1 > $hi2 } \
   {
	set hi $hi1
   } \
   else \
   {
	set hi $hi2
   }

   # Adjust the position of the text wi1 with an offset.
   if { $hi1 < $hi2 } \
   {
	set offset [expr $hi2 - $hi1]
   } \
   else \
   {
	set offset 0
   }
   set wi1 [image width b_dir_remote]
   incr wi1 4
   set wi2 [winfo width .dirsel.f2.cv]

   # Read user-defined and automatically generated text files
   # containing a list of public groups and the names of the
   # machines that they are on.
   set list_of_groups_and_machine_names [list]

   set inputOperator \
	[open "|cat $env(MCLITE_INSTALL)/.sysdata/mclite_remote_user" r]
   while {[gets $inputOperator line] >= 0} \
   {	
	if {[string compare [string range $line 0 0] "#"] != 0} \
	{
	   lappend list_of_groups_and_machine_names $line
	}
   }
   catch {close $inputOperator}

   set inputOperator \
	[open "|cat $env(MCLITE_INSTALL)/.sysdata/mclite_remote_auto" r]
   while {[gets $inputOperator line] >= 0} \
   {	
	if {[string compare [string range $line 0 0] "#"] != 0} \
	{
	   lappend list_of_groups_and_machine_names $line
	}
   }
   catch {close $inputOperator}
   
   # Clear the canvas
   .dirsel.f2.cv delete all

   # Place directories into list box
   foreach file $list_of_groups_and_machine_names \
   { 
	# Add to McIDAS routing table
	mcidas exec "DATALOC ADD [lindex $file 0] [lindex $file 1] DEV=NNN"

	set image_file b_dir_remote
	set image_tag REMOTE_GROUP_IMG
	set text_tag REMOTE_GROUP_TXT

	.dirsel.f2.cv create image 2 [expr $ind * $hi] \
	    -anchor nw \
	    -image $image_file \
	    -tags $image_tag
	.dirsel.f2.cv create text $wi1 [expr ($ind * $hi) + $offset] \
	    -anchor nw \
	    -text "[format "%-12.12s" [lindex $file 0]] \
			[string range $file [string first "(" $file] \
			[string first ")" $file]]" \
	    -fill $tkgetdir::colors(lstfg) \
	    -tags $text_tag \
	    -font $tkgetdir::listfont

	set ind [ expr $ind + 1 ]
   }

    set ha [expr $ind * $hi]
    .dirsel.f2.cv configure -scrollregion [list 0 0 $wi2 $ha]
    .dirsel.f2.cv yview moveto 0.0
}


###########################################################
proc tkgetdir::DoubleClickRemoteGroup {input_group} \
{
   global current_directory
   global initial_directory
   global current_group

   # Internet speed could slow things down so set cursor to a watch
   showBusyStatus

   # Set the group name
   if {[string compare $input_group "X"] == 0} \
   {
	set id [.dirsel.f2.cv find withtag current]

	# If the folder image was clicked on, increment to next item
	set type [.dirsel.f2.cv type $id]
   	if {[string compare $type "image"] == 0} {incr id}

   	# Get the name of the new directory
   	set current_group [lindex [.dirsel.f2.cv itemcget $id -text] 0]

 	set temp_path "[.dirsel.f1.f1a.directoryWindow cget -text]/$current_group"

   } \
   else \
   {
	set current_group $input_group
	set temp_path "$initial_directory/RemoteData/$input_group"
   }

   # Add the new directory to the path in the directory window
   .dirsel.f1.f1a.directoryWindow configure \
	-text $temp_path

   set current_directory $temp_path

   # Call DSINFO to get a list of datasets for the given group
   set inputOperator [open "|dsinfo.k I $current_group" r]

   # Loop through first five lines checking for error messages, if
   # this is an accessible group then the first five lines are just
   # header lines anyway.
   for {set i 1} {$i <= 5} {incr i} \
   {
	gets $inputOperator line
	if {[string compare [lindex $line 0] "No"] == 0 || \
            [string compare [lindex $line 1] "Accounting"] == 0} \
   	{
	   showIdleStatus; # Return cursor to normal
  	   .dirsel.f2.cv delete all; # Clear the canvas
	   tk_messageBox \
	   	-message "This group is empty,\
				or it is not publicly accessible, \
				or it does not exist on the remote server." \
	   	-icon warning \
	   	-title "Warning"
	   return
   	}
   }

   # Group is OK so read list of datasets
   set dataset_list [list]
   while {[gets $inputOperator line] >= 0} \
   {
	if {[string compare $line "DSINFO -- done"] != 0 && \
	    [string length $line]} \
	{
	   lappend dataset_list [lindex $line 0]
	   set dataset_description([lindex $line 0]) [string range $line 22 end]
	}
   }
   catch {close $inputOperator}

   set ind 0
   set hi1 [font metrics $tkgetdir::myfont -linespace]
   set hi2 [image height b_dir_remote]
   if { $hi1 > $hi2 } \
   {
	set hi $hi1
   } \
   else \
   {
	set hi $hi2
   }

   # Adjust the position of the text wi1 with an offset.
   if { $hi1 < $hi2 } \
   {
	set offset [expr $hi2 - $hi1]
   } \
   else \
   {
	set offset 0
   }
   set wi1 [image width b_dir_remote]
   incr wi1 4
   set wi2 [winfo width .dirsel.f2.cv]

   # Clear the canvas
   .dirsel.f2.cv delete all

   set image_file b_dir_remote
   set image_tag REMOTE_DATASET_IMG
   set text_tag REMOTE_DATASET_TXT

   # Place directories into list box
   foreach dataset $dataset_list \
   { 
	.dirsel.f2.cv create image 2 [expr $ind * $hi] \
	    -anchor nw \
	    -image $image_file \
	    -tags $image_tag
	.dirsel.f2.cv create text $wi1 [expr ($ind * $hi) + $offset] \
	    -anchor nw \
	    -text "[format "%-12.12s" $dataset] \
			($dataset_description($dataset))" \
	    -fill $tkgetdir::colors(lstfg) \
	    -tags $text_tag \
	    -font $tkgetdir::listfont

	set ind [ expr $ind + 1 ]
   }

    set ha [expr $ind * $hi]
    .dirsel.f2.cv configure -scrollregion [list 0 0 $wi2 $ha]
    .dirsel.f2.cv yview moveto 0.0

   # Return cursor to normal
   showIdleStatus

}

###########################################################
proc tkgetdir::DoubleClickRemoteDataset {} \
{
   global current_directory
   global current_group
   global virtual_file_array
   global num_images
   global remote_flag

   # Internet speed could slow things down so set cursor to a watch
   showBusyStatus

   # Set 'remote_flag' to REMOTE since we are viewing a list
   # of remote image files.
   set remote_flag REMOTE

   set id [.dirsel.f2.cv find withtag current]

   # If the folder image was clicked on, increment to next item
   set type [.dirsel.f2.cv type $id]
   if {[string compare $type "image"] == 0} {incr id}

   # Get the name of the new directory
   set nam [lindex [.dirsel.f2.cv itemcget $id -text] 0]
   set directory_name $nam

   # Add the new directory to the path in the directory window
   set temp_path [.dirsel.f1.f1a.directoryWindow cget -text]
   .dirsel.f1.f1a.directoryWindow configure \
	-text "$temp_path/$nam"

   set current_directory $temp_path/$nam

   # Call IMGLIST to get a list of image files for the given dataset
   set inputOperator [open "|imglist.k $current_group/$nam.-99" r]

   # Burn header lines
   gets $inputOperator line
   gets $inputOperator line
   gets $inputOperator line
   gets $inputOperator line

   # Read output from IMGLIST
   set counter 0
   set temp_list [list]
   array set virtual_file_array {}
   while {[gets $inputOperator line] >= 0} \
   {
	# Each element in the 'virtual_file_array' array will contain;
	# 	the newly created virtual file name (data & time of image),
	# 	the group name,
	#       the dataset name,
	#	the absolute position within the dataset.
	if {[string compare $line "imglist.k: done"] != 0} \
	{
	   incr counter
	   
	   # Create virtual file name
	   set temp1 [string range $line 20 21]
	   set temp2 [string range $line 23 25]
	   set temp3 [string range $line 27 31]
	   set temp4 [string range $line 34 41]
	   set virtual_file_name "$temp1\_$temp2\_$temp3\_$temp4"

	   set temp_list \
	   	"$virtual_file_name \
	   	$current_group \
	   	$nam \
	   	[lindex $line 0]"

	   set virtual_file_array($counter) $temp_list
	}
   }
   catch {close $inputOperator}

   set num_images $counter

   set ind 0
   set hi1 [font metrics $tkgetdir::myfont -linespace]
   set hi2 [image height b_dir_remote]
   if { $hi1 > $hi2 } \
   {
	set hi $hi1
   } \
   else \
   {
	set hi $hi2
   }

   # Adjust the position of the text wi1 with an offset.
   if { $hi1 < $hi2 } \
   {
	set offset [expr $hi2 - $hi1]
   } \
   else \
   {
	set offset 0
   }
   set wi1 [image width b_dir_remote]
   incr wi1 4
   set wi2 [winfo width .dirsel.f2.cv]

   # Clear the canvas
   .dirsel.f2.cv delete all

   set text_tag REMOTE_IMAGEFILE_TXT

   # Place image files into list box
   for {set i 1} {$i <= $num_images} {incr i} \
   { 
	.dirsel.f2.cv create text 5 [expr ($ind * $hi) + $offset] \
	    -anchor nw \
	    -text [lindex $virtual_file_array($i) 0] \
	    -fill $tkgetdir::colors(lstfg) \
	    -tags $text_tag \
	    -font $tkgetdir::listfont

	set ind [ expr $ind + 1 ]
   }

    set ha [expr $ind * $hi]
    .dirsel.f2.cv configure -scrollregion [list 0 0 $wi2 $ha]
    .dirsel.f2.cv yview moveto 0.0

   # Return cursor to normal
   showIdleStatus

}

#############################################################
# tkgetdir::DoubleClickRemoteImageFile {} 
#############################################################
proc tkgetdir::DoubleClickRemoteImageFile {} \
{
   global virtual_file_array
   global num_images

   set id [.dirsel.f2.cv find withtag current_item]

   if {[string length $id] == 0} \
   {
	tk_messageBox \
	   -message "You must select a file first." \
	   -icon error \
	   -title "Error"
	return
   }

   # Get the name of the new directory
   set nam [.dirsel.f2.cv itemcget $id -text]

   # Determine which image to use
   for {set i 1} {$i <= $num_images} {incr i} \
   {
	if {[string compare [lindex $virtual_file_array($i) 0] $nam] == 0} \
	{
	   set temp_file $virtual_file_array($i)
	   break
	}
   }

   # Kill image property window if it already exists
   if { [winfo exists .tlImageSelect] } {destroy .tlImageSelect}

   # Send the dataset and position to ImageSelect
   ImageSelect [lindex $temp_file 0] \
	       [lindex $temp_file 1] \
	       [lindex $temp_file 2] \
	       [lindex $temp_file 3]
}



##################################################################################
# FileSelector {old_directory} - This process takes in the current directory
#				 and returns the new directory from the
#				 directory selection GUI.
##################################################################################
proc FileSelector { old_directory } \
{
	global data
	global env


   	# Make existing window reappear if called again   
     	if { [winfo exists .dirsel] } \
	{ 
		wm deiconify .dirsel; \
		focus .dirsel; \
		raise .dirsel; \
		return
	}

	# Instantiate new directory selector window
	set new_directory \
	   [tkgetdir::tk_getDirectory \
	      -initialdir $old_directory \
	      -title "File Selector" \
	      -font "-schumacher-clean-bold-r-normal--13-*-*-*-c-60-iso8859-1" \
	      -listfont \
		"-schumacher-clean-bold-r-normal--13-*-*-*-c-60-iso8859-1" \
	      -ok "OK" \
	      -cancel "Dismiss" \
	      -geometry 400x250+648+0 \
	      -listbg #FFFFFD \
	      -hilightfg white \
	      -hilightbg blue \
	      -selectfg white \
	      -selectbg navy]

	return $new_directory
}

