# Copyright(c) 1999, Space Science and Engineering Center, UW-Madison
# Refer to "McIDAS Software Acquisition and Distribution Policies"
# in the file  mcidas/data/license.txt

# $Id: ProgressBar.tcl,v 1.4 1999/05/24 18:36:13 beckys Rel $

########################################################################
# This module is used to pop up a progress window while data is being  #
# processed in the GUI.  					       #
########################################################################
#
# GlobalProcedures
#
#	ProgressInit -- Used to set up the progress window
#	ProgressUpdate -- updates the progress window "bar"
#	ProgressClose -- closes the progress window
#
#  Global Variables
#
#	ProgressWindow -- name of the progress window
#	ProgressBarCount -- Current count/interval for displaying
#	ProgressBarDirection -- bar moving left(0) or right(1)
#
########################################################################
proc ProgressInit { Label } \
  {
  global ProgressWindow
  global ProgressBarCount
  global ProgressBarDirection
  global ProgressBarRect

  toplevel .tlProgress -relief raised -borderwidth 3
  wm title .tlProgress "Activity Meter"
  
  label .tlProgress.lbCaption -text $Label
  pack .tlProgress.lbCaption -side top -padx 10 -pady 5
  frame .tlProgress.fmBar -relief sunken -borderwidth 3
    canvas .tlProgress.fmBar.cvBar -width 300 -height 20
    pack .tlProgress.fmBar.cvBar -side top
  pack .tlProgress.fmBar -side top -pady 5 -padx 10

  .tlProgress.fmBar.cvBar create rect 0 0 299 19 -fill grey -outline grey
  
  set ProgressWindow .tlProgress
  set ProgressBarCount -1
  set ProgressBarDirection 1

  # -- create the rectangle and just move it with each update
  set ProgressBarRect [$ProgressWindow.fmBar.cvBar create rect 0 0 19 19 -fill white -outline black]
  #--- time for the watch cursor
    showBusyStatus 
  }
#------------------------------------------------------------------------#
# ProgressUpdate -- routine to update the position of the progress bar   #
#------------------------------------------------------------------------#  
proc ProgressUpdate  { } \
  {
  global ProgressWindow
  global ProgressBarCount
  global ProgressBarDirection
  global ProgressBarRect 
  #---verify that a window is there
    if { !([info exists ProgressWindow ] && [string length $ProgressWindow]) } \
      {
      return
      } 
    if { $ProgressBarDirection==1} \
      {
      #---moving right
        set Newx 10
        incr ProgressBarCount
	if { $ProgressBarCount==28 } \
	  {
	  set ProgressBarDirection 0
	  set Newx -10
	  }
      } \
    else \
      {  
      #--- moving left  
        set Newx -10
        incr ProgressBarCount -1
	if { $ProgressBarCount==0 } \
	  { 
	  set ProgressBarDirection 1 
	  set Newx 10
	  }
      }    
  #---Time to draw the bar
    $ProgressWindow.fmBar.cvBar move $ProgressBarRect $Newx 0 
    update ;#idletasks
  }
  
#------------------------------------------------------------------------#
# ProgressClose -- remove the window and restore the cursor              #
#------------------------------------------------------------------------#  
proc ProgressClose { } \
  {
  global ProgressWindow
  
  showIdleStatus   
  destroy $ProgressWindow
  }
