#!/bin/false

# Copyright(c) 1998, Space Science and Engineering Center, UW-Madison
# Refer to "McIDAS Software Acquisition and Distribution Policies"
# in the file  mcidas/data/license.txt

# $Id: initbrief.tcl,v 1.7 1998/10/13 20:26:37 dglo Exp $

# look for briefing frames starting at root window
#
proc findMcBrief {instance} {
  # get hostname
  #
  if [catch {set hostname [exec hostname 2>/dev/null]}] {
    set hostname "unknown"
  }

  # try to find the best possible image window
  #
  set thishost ""
  set thisinst ""
  foreach id [xinfo find -name "mcimage" -class "McImage" -depth 3] {
    if {[catch {set inm [xinfo iconname $id]}] == 0} {
      regexp "^(.*):(.+)@(.+)\$" $inm full inst user host
      if {$host == $hostname && $inst == $instance} {
	if {$thishost != ""} {
	  puts stderr "Sorry, more than one briefing frame displayed from \"$hostname\""
	} else {
	  set thishost $id
	}
      } elseif {$inst == $instance} {
	if {$thisinst != ""} {
	  puts stderr "Sorry, more than one briefing frame displayed"
	} else {
	  set thisinst $id
	}
      }
    }
  }

  if {$thishost != ""} {
    return $thishost
  }
  if {$thisinst != ""} {
    return $thisinst
  }

  puts stderr "Sorry, couldn't find the appropriate briefing frame"
  exit 1
}

# attempt to grab a McIDAS briefing frame and subsume it into a Tk window
#
proc initBrief {widget instance} {

  # get a list of McIDAS briefing frames
  #
  set briefid [findMcBrief $instance]

  # figure out how large the frame needs to be
  #
  set stuff [xinfo geometry $briefid]
  regexp {([0-9]+)x([0-9]+)} $stuff full width height

  # build and pack the frame
  #
  frame $widget -relief flat -background black -borderwidth 0 \
      -width $width -height $height
  pack $widget -expand 1 -fill both
  update idletasks

  # grab the briefing frame
  #
  set xwidget $widget.brief
  xwindow $xwidget $briefid
#  $xwidget configure -borderwidth 0
  pack $xwidget -expand 1 -fill both

  # if briefing frame configures itself, undo any user-specified configuration
  #
  set top [winfo toplevel $widget]
  bind $xwidget <Configure> "wm geometry $top {}"

  # if briefing frame destroys itself, kill parent
  #
  bind $xwidget <Destroy> "killkids; destroy $top"

  # set title and icon name to those of the briefing frame
  #
  wm iconname $top [xinfo iconname $briefid]
  wm title $top [xinfo title $briefid]

  return $xwidget
}
