# Copyright(c) 1998, Space Science and Engineering Center, UW-Madison
# Refer to "McIDAS Software Acquisition and Distribution Policies"
# in the file  mcidas/data/license.txt

# $Id: widgetTools.tcl,v 1.5 1998/10/13 20:26:54 dglo Exp $

# widgetTools		Handy "tools", i.e. procs for doing things to
#			widgets.
#

#========================================================================
#
#	Disabling and Enabling Labels
#
#========================================================================


proc disableLabel {w} {
#
# disableLabel		"Disable" a label, i.e. change its foreground
#			color to something similar to disabled buttons
#			and menus.
#
# Arguments:
#
#	w		The name of the label widget to disable.
#
	$w configure -foreground gray80
}


proc enableLabel {w} {
#
# enableLabel		"Enable" a label, i.e. change its foreground
#			color to the normal foreground color.
#
# Arguments:
#
#	w		The name of the label widget to enable.
#
	set color null
	set color [option get $w foreground Foreground]
	if { $color != "null" } {
		$w configure -foreground $color
	} else {
		puts "enableLable: ERROR can't get foreground color."
	}
}





#========================================================================
#
#	Up and Down Arrow Widgets.
#
#========================================================================


proc mkUpDown {w txt} {
#
# mkUpDown		Make a set of widgets containing a label, an entry,
#			and up and down arrows to adjust the values in the
#			entry.
# Arguments:
#	w		The name of the widget to hang these subwidgets on.
#	txt		Text for the label.
# Globals:
#	guipath		Path to gui directory where bitmaps are.
#	UDval		Chosen value.
#	UDvalues	List of valid values.
#	UDindex		Index of chosen value in array.
#
	global guipath
	global UDval UDindex UDvalues

	frame $w
	label $w.label -text $txt
	entry $w.entry -textvariable UDval -width 10 -relief sunken \
		-state disabled
	button $w.uparr -bitmap @[bitmapLoc uparrow.xbm] -command incUD
	button $w.downarr -bitmap @[bitmapLoc downarrow.xbm] -command decUD

	pack	$w.label	-side left
	pack	$w.entry	-side right
	pack	$w.uparr	-side right
	pack	$w.downarr	-side right
	pack	$w
}

proc incUD {} {
#
# incUD			Increment the UDindex and put the corresponding
#			value in UDval
#
	global UDval UDindex UDvalues

	set length [llength $UDvalues]
	if { $UDindex < [expr $length - 1] } {
		set UDindex [expr {$UDindex + 1}]
		set UDval [lindex $UDvalues $UDindex]
	}
}


proc decUD {} {
#
# decUD			Decrement the UDindex and put the corresponding
#			value in UDval
#
	global UDval UDindex UDvalues

	if { $UDindex > 0 } {
		set UDindex [expr {$UDindex - 1}]
		set UDval [lindex $UDvalues $UDindex]
	}
}
